@@ -30,9 +30,12 @@ log = logging.getLogger(__name__)
 log.setLevel(logging.DEBUG)
 
 
+_MAX_SIP_DEGREE = 6
+
+
 __all__ = ["reproject", "wcs_from_footprints", "velocity_correction",
            "MSAFileError", "NoDataOnDetectorError", "compute_scale",
-           "calc_rotation_matrix", "wrap_ra"]
+           "calc_rotation_matrix", "wrap_ra", "update_fits_wcsinfo"]
 
 
 class MSAFileError(Exception):
@@ -132,7 +135,7 @@ def compute_scale(wcs: WCS, fiducial: Union[tuple, np.ndarray],
     delta[spatial_idx[0]] = 1
 
     crpix_with_offsets = np.vstack((crpix, crpix + delta, crpix + np.roll(delta, 1))).T
-    crval_with_offsets = wcs(*crpix_with_offsets)
+    crval_with_offsets = wcs(*crpix_with_offsets, with_bounding_box=False)
 
     coords = SkyCoord(ra=crval_with_offsets[spatial_idx[0]], dec=crval_with_offsets[spatial_idx[1]], unit="deg")
     xscale = np.abs(coords[0].separation(coords[1]).value)
@@ -278,7 +281,7 @@ def wcs_from_footprints(dmodels, refmodel=None, transform=None, bounding_box=Non
                 fiducial[k] = crval[i]
                 i += 1
 
-    ref_fiducial = compute_fiducial([refmodel.meta.wcs])
+    ref_fiducial = np.array([refmodel.meta.wcsinfo.ra_ref, refmodel.meta.wcsinfo.dec_ref])
 
     prj = astmodels.Pix2Sky_TAN()
 
@@ -315,15 +318,14 @@ def wcs_from_footprints(dmodels, refmodel=None, transform=None, bounding_box=Non
             transform = functools.reduce(lambda x, y: x | y, transform)
 
     out_frame = refmodel.meta.wcs.output_frame
-    input_frame = dmodels[0].meta.wcs.input_frame
+    input_frame = refmodel.meta.wcs.input_frame
     wnew = wcs_from_fiducial(fiducial, coordinate_frame=out_frame, projection=prj,
                              transform=transform, input_frame=input_frame)
 
     footprints = [w.footprint().T for w in wcslist]
     domain_bounds = np.hstack([wnew.backward_transform(*f) for f in footprints])
-
-    for axs in domain_bounds:
-        axs -= (axs.min() + .5)
+    axis_min_values = np.min(domain_bounds, axis=1)
+    domain_bounds = (domain_bounds.T - axis_min_values).T
 
     output_bounding_box = []
     for axis in out_frame.axes_order:
@@ -332,9 +334,9 @@ def wcs_from_footprints(dmodels, refmodel=None, transform=None, bounding_box=Non
 
     output_bounding_box = tuple(output_bounding_box)
     if crpix is None:
-        ax1, ax2 = np.array(output_bounding_box)[sky_axes]
-        offset1 = (ax1[1] - ax1[0]) / 2
-        offset2 = (ax2[1] - ax2[0]) / 2
+        offset1, offset2 = wnew.backward_transform(*fiducial)
+        offset1 -= axis_min_values[0]
+        offset2 -= axis_min_values[1]
     else:
         offset1, offset2 = crpix
     offsets = astmodels.Shift(-offset1, name='crpix1') & astmodels.Shift(-offset2, name='crpix2')
@@ -370,11 +372,15 @@ def compute_fiducial(wcslist, bounding_box=None):
     if spatial_footprint.any():
         lon, lat = spatial_footprint
         lon, lat = np.deg2rad(lon), np.deg2rad(lat)
-        x_mean = np.mean(np.cos(lat) * np.cos(lon))
-        y_mean = np.mean(np.cos(lat) * np.sin(lon))
-        z_mean = np.mean(np.sin(lat))
-        lon_fiducial = np.rad2deg(np.arctan2(y_mean, x_mean)) % 360.0
-        lat_fiducial = np.rad2deg(np.arctan2(z_mean, np.sqrt(x_mean ** 2 + y_mean ** 2)))
+        x = np.cos(lat) * np.cos(lon)
+        y = np.cos(lat) * np.sin(lon)
+        z = np.sin(lat)
+
+        x_mid = (np.max(x) + np.min(x)) / 2.
+        y_mid = (np.max(y) + np.min(y)) / 2.
+        z_mid = (np.max(z) + np.min(z)) / 2.
+        lon_fiducial = np.rad2deg(np.arctan2(y_mid, x_mid)) % 360.0
+        lat_fiducial = np.rad2deg(np.arctan2(z_mid, np.sqrt(x_mid ** 2 + y_mid ** 2)))
         fiducial[spatial_axes] = lon_fiducial, lat_fiducial
     if spectral_footprint.any():
         fiducial[spectral_axes] = spectral_footprint.min()
@@ -568,10 +574,11 @@ def get_object_info(catalog_name=None):
 
 def create_grism_bbox(input_model,
                       reference_files=None,
-                      mmag_extract=99.0,
+                      mmag_extract=None,
                       extract_orders=None,
                       wfss_extract_half_height=None,
-                      wavelength_range=None):
+                      wavelength_range=None,
+                      nbright=None):
     """Create bounding boxes for each object in the catalog
 
     The sky coordinates in the catalog image are first related
@@ -605,6 +612,8 @@ def create_grism_bbox(input_model,
     wavelength_range : dict, optional
         Pairs of {spectral_order: (wave_min, wave_max)} for each order.
         If ``None``, the default one in the wavelengthrange reference file is used.
+    nbright : int, optional
+        The number of brightest objects to extract from the catalog.
 
     Returns
     -------
@@ -663,9 +672,12 @@ def create_grism_bbox(input_model,
 
             wavelength_range = f.get_wfss_wavelength_range(filter_name, extract_orders)
 
+    if mmag_extract is None:
+        mmag_extract = 999.  # extract all objects, regardless of magnitude
+    else:
+        log.info("Extracting objects < abmag = {0}".format(mmag_extract))
     if not isinstance(mmag_extract, (int, float)):
         raise TypeError(f"Expected mmag_extract to be a number, got {mmag_extract}")
-    log.info("Extracting objects < abmag = {0}".format(mmag_extract))
 
     # extract the catalog objects
     if input_model.meta.source_catalog is None:
@@ -675,11 +687,12 @@ def create_grism_bbox(input_model,
 
     log.info(f"Getting objects from {input_model.meta.source_catalog}")
 
-    return _create_grism_bbox(input_model, mmag_extract, wfss_extract_half_height, wavelength_range)
+    return _create_grism_bbox(input_model, mmag_extract, wfss_extract_half_height, wavelength_range,
+                              nbright)
 
 
-def _create_grism_bbox(input_model, mmag_extract=99.0,
-                       wfss_extract_half_height=None, wavelength_range=None):
+def _create_grism_bbox(input_model, mmag_extract=None, wfss_extract_half_height=None,
+                       wavelength_range=None, nbright=None):
 
     log.debug(f'Extracting with wavelength_range {wavelength_range}')
 
@@ -803,7 +816,7 @@ def _create_grism_bbox(input_model, mmag_extract=99.0,
                             log.info("Partial order on detector for obj: {} order: {}".format(obj.label, order))
 
                     if not exclude:
-                        order_bounding[order] = ((round(ymin), round(ymax)), (round(xmin), round(xmax)))
+                        order_bounding[order] = ((ymin, ymax), (xmin, xmax))
                         waverange[order] = ((lmin, lmax))
                         partial_order[order] = ispartial
 
@@ -819,10 +832,31 @@ def _create_grism_bbox(input_model, mmag_extract=99.0,
                                                      sky_bbox_ur=obj.sky_bbox_ur,
                                                      xcentroid=xcenter,
                                                      ycentroid=ycenter,
-                                                     is_extended=obj.is_extended))
-    if len(grism_objects) == 0:
-        log.warning("No grism objects saved, check catalog")
-    return grism_objects
+                                                     is_extended=obj.is_extended,
+                                                     isophotal_abmag=obj.isophotal_abmag))
+
+    # At this point we have a list of grism objects limited to
+    # isophotal_abmag < mmag_extract. We now need to further restrict
+    # the list to the N brightest objects, as given by nbright.
+    if nbright is None:
+        # Include all objects, regardless of brightness
+        final_objects = grism_objects
+    else:
+        # grism_objects is a list of objects, so it's not easy or practical
+        # to sort it directly. So create a list of the isophotal_abmags, which
+        # we'll then use to find the N brightest objects.
+        indxs = np.argsort([obj.isophotal_abmag for obj in grism_objects])
+
+        # Create a final grism object list containing only the N brightest objects
+        final_objects = []
+        final_objects = [grism_objects[i] for i in indxs[:nbright]]
+        del grism_objects
+
+    log.info(f"Total of {len(final_objects)} grism objects defined")
+    if len(final_objects) == 0:
+        log.warning("No grism objects saved; check catalog or step params")
+
+    return final_objects
 
 
 def get_num_msa_open_shutters(shutter_state):
@@ -960,9 +994,9 @@ def compute_footprint_spectral(model):
     max_ra = np.nanmax(ra)
 
     # for the footprint we want the ra values to fall between 0 to 360
-    if(min_ra < 0):
+    if min_ra < 0:
         min_ra = min_ra + 360.0
-    if(max_ra >= 360.0):
+    if max_ra >= 360.0:
         max_ra = max_ra - 360.0
     footprint = np.array([[min_ra, np.nanmin(dec)],
                           [max_ra, np.nanmin(dec)],
@@ -1089,9 +1123,9 @@ def compute_footprint_nrs_ifu(dmodel, mod):
     ra_max = np.nanmax(ra_total)
     ra_min = np.nanmin(ra_total)
     # for the footprint we want ra to be between 0 to 360
-    if(ra_min < 0):
+    if ra_min < 0:
         ra_min = ra_min + 360.0
-    if(ra_max >= 360.0):
+    if ra_max >= 360.0:
         ra_max = ra_max - 360.0
 
     dec_max = np.nanmax(dec_total)
@@ -1175,10 +1209,10 @@ def wrap_ra(ravalues):
     nwrap = wrap_index[0].size
 
     # get all the ra on the same "side" of 0/360
-    if(nwrap != 0 and median_ra < 180):
+    if nwrap != 0 and median_ra < 180:
         ravalues_wrap[wrap_index] = ravalues_wrap[wrap_index] - 360.0
 
-    if(nwrap != 0 and median_ra > 180):
+    if nwrap != 0 and median_ra > 180:
         ravalues_wrap[wrap_index] = ravalues_wrap[wrap_index] + 360.0
 
     # if the input ravaules are a list - return a list
@@ -1212,3 +1246,177 @@ def in_ifu_slice(slice_wcs, ra, dec, lam):
     onslice_ind = np.isclose(slx, SLX,atol=5e-4)
 
     return onslice_ind
+
+
+def update_fits_wcsinfo(datamodel, max_pix_error=0.01, degree=None, npoints=32,
+                        crpix=None, projection='TAN', **kwargs):
+    """
+    For imaging data models *only*, update data model's ``meta.wcsinfo``
+    attribute using a FITS SIP approximation of the current data model's
+    GWCS from ``meta.wcs``.
+
+    The default mode in using this attempts to achieve roughly 0.01 pixel
+    accuracy over the entire image.
+
+    This function uses the :py:meth:`~gwcs.wcs.WCS.to_fits_sip` to
+    create FITS WCS representations of GWCS objects. Only most important
+    :py:meth:`~gwcs.wcs.WCS.to_fits_sip` parameters are exposed here. Other
+    arguments to :py:meth:`~gwcs.wcs.WCS.to_fits_sip` can be passed via
+    ``kwargs`` - see "Other Parameters" section below.
+    Please refer to the documentation of :py:meth:`~gwcs.wcs.WCS.to_fits_sip`
+    for more details.
+
+    .. warning::
+        This function modifies input data model's ``datamodel.meta.wcsinfo``
+        members.
+
+
+    Parameters
+    ----------
+
+    max_pix_error : float, optional
+        Maximum allowed error over the domain of the pixel array. This
+        error is the equivalent pixel error that corresponds to the maximum
+        error in the output coordinate resulting from the fit based on
+        a nominal plate scale.
+
+    degree : int, iterable, None, optional
+        Degree of the SIP polynomial. Default value `None` indicates that
+        all allowed degree values (``[1...6]``) will be considered and
+        the lowest degree that meets accuracy requerements set by
+        ``max_pix_error`` will be returned. Alternatively, ``degree`` can be
+        an iterable containing allowed values for the SIP polynomial degree.
+        This option is similar to default `None` but it allows caller to
+        restrict the range of allowed SIP degrees used for fitting.
+        Finally, ``degree`` can be an integer indicating the exact SIP degree
+        to be fit to the WCS transformation. In this case
+        ``max_pixel_error`` is ignored.
+
+    npoints : int, optional
+        The number of points in each dimension to sample the bounding box
+        for use in the SIP fit. Minimum number of points is 3.
+
+    crpix : list of float, None, optional
+        Coordinates (1-based) of the reference point for the new FITS WCS.
+        When not provided, i.e., when set to `None` (default) the reference
+        pixel already specified in ``wcsinfo`` will be re-used. If
+        ``wcsinfo`` does not contain ``crpix`` information, then the
+        reference pixel will be chosen near the center of the bounding box
+        for axes corresponding to the celestial frame.
+
+    projection : str, `~astropy.modeling.projections.Pix2SkyProjection`, optional
+        Projection to be used for the created FITS WCS. It can be specified
+        as a string of three characters specifying a FITS projection code
+        from Table 13 in
+        `Representations of World Coordinates in FITS \
+        <https://doi.org/10.1051/0004-6361:20021326>`_
+        (Paper I), Greisen, E. W., and Calabretta, M. R., A & A, 395,
+        1061-1075, 2002. Alternatively, it can be an instance of one of the
+        `astropy's Pix2Sky_* <https://docs.astropy.org/en/stable/modeling/\
+        reference_api.html#module-astropy.modeling.projections>`_
+        projection models inherited from
+        :py:class:`~astropy.modeling.projections.Pix2SkyProjection`.
+
+
+    Other Parameters
+    ----------------
+
+    max_inv_pix_error : float, None, optional
+        Maximum allowed inverse error over the domain of the pixel array
+        in pixel units. With the default value of `None` no inverse
+        is generated.
+
+    inv_degree : int, iterable, None, optional
+        Degree of the SIP polynomial. Default value `None` indicates that
+        all allowed degree values (``[1...6]``) will be considered and
+        the lowest degree that meets accuracy requerements set by
+        ``max_pix_error`` will be returned. Alternatively, ``degree`` can be
+        an iterable containing allowed values for the SIP polynomial degree.
+        This option is similar to default `None` but it allows caller to
+        restrict the range of allowed SIP degrees used for fitting.
+        Finally, ``degree`` can be an integer indicating the exact SIP degree
+        to be fit to the WCS transformation. In this case
+        ``max_inv_pixel_error`` is ignored.
+
+    bounding_box : tuple, None, optional
+        A pair of tuples, each consisting of two numbers
+        Represents the range of pixel values in both dimensions
+        ((xmin, xmax), (ymin, ymax))
+
+    verbose : bool, optional
+        Print progress of fits.
+
+
+    Returns
+    -------
+    FITS header with all SIP WCS keywords
+
+
+    Raises
+    ------
+    ValueError
+        If the WCS is not at least 2D, an exception will be raised. If the
+        specified accuracy (both forward and inverse, both rms and maximum)
+        is not achieved an exception will be raised.
+
+
+    Notes
+    -----
+
+    Use of this requires a judicious choice of required accuracies.
+    Attempts to use higher degrees (~7 or higher) will typically fail due
+    to floating point problems that arise with high powers.
+
+    For more details, see :py:meth:`~gwcs.wcs.WCS.to_fits_sip`.
+
+    """
+    # make a copy of kwargs:
+    kwargs = {k: v for k, v in kwargs.items()}
+
+    # override default values for "other parameters":
+    max_inv_pix_error = kwargs.pop('max_inv_pix_error', None)
+    inv_degree = kwargs.pop('inv_degree', None)
+    if inv_degree is None:
+        inv_degree = range(1, _MAX_SIP_DEGREE)
+
+    # limit default 'degree' range to _MAX_SIP_DEGREE:
+    if degree is None:
+        degree = range(1, _MAX_SIP_DEGREE)
+
+    if crpix is None:
+        crpix = [datamodel.meta.wcsinfo.crpix1, datamodel.meta.wcsinfo.crpix2]
+    if None in crpix:
+        crpix = None
+
+    hdr = datamodel.meta.wcs.to_fits_sip(
+        max_pix_error=max_pix_error,
+        degree=degree,
+        max_inv_pix_error=max_inv_pix_error,
+        inv_degree=inv_degree,
+        npoints=npoints,
+        crpix=crpix,
+        **kwargs
+    )
+
+    # update meta.wcs_info with fit keywords except for naxis*
+    del hdr['naxis*']
+
+    # maintain convention of lowercase keys
+    hdr_dict = {k.lower(): v for k, v in hdr.items()}
+
+    # delete naxis, cdelt, pc from wcsinfo
+    rm_keys = ['naxis', 'cdelt1', 'cdelt2',
+               'pc1_1', 'pc1_2', 'pc2_1', 'pc2_2',
+               'a_order', 'b_order', 'ap_order', 'bp_order']
+
+    rm_keys.extend(f"{s}_{i}_{j}" for i in range(10) for j in range(10)
+                   for s in ['a', 'b', 'ap', 'bp'])
+
+    for key in rm_keys:
+        if key in datamodel.meta.wcsinfo.instance:
+            del datamodel.meta.wcsinfo.instance[key]
+
+    # update meta.wcs_info with fit keywords
+    datamodel.meta.wcsinfo.instance.update(hdr_dict)
+
+    return hdr
