@@ -14,6 +14,7 @@ from .utilities import (
     getattr_from_list,
     is_iterable
 )
+from ..pool import PoolRow
 
 __all__ = [
     'AttrConstraint',
@@ -303,8 +304,8 @@ class SimpleConstraint(SimpleConstraintABC):
 
         # Determine reprocessing
         reprocess = []
-        if (self.matched and self.reprocess_on_match) or \
-           (not self.matched and self.reprocess_on_fail):
+        if ((self.matched and self.reprocess_on_match) or
+                (not self.matched and self.reprocess_on_fail)):
             reprocess.append(ProcessList(
                 items=[item],
                 work_over=self.work_over,
@@ -441,7 +442,7 @@ class AttrConstraint(SimpleConstraintABC):
                     )
                 )
             self.matched = True
-            return (self.matched, reprocess)
+            return self.matched, reprocess
 
         # Get the condition information.
         try:
@@ -468,7 +469,7 @@ class AttrConstraint(SimpleConstraintABC):
             if is_iterable(evaled):
                 reprocess_items = []
                 for avalue in evaled:
-                    new_item = deepcopy(item)
+                    new_item = PoolRow(item)
                     new_item[source] = str(avalue)
                     reprocess_items.append(new_item)
                 reprocess.append(ProcessList(
@@ -647,8 +648,8 @@ class Constraint:
         self.matched, reprocess = self.reduce(item, self.constraints)
 
         # Determine reprocessing
-        if (self.matched and self.reprocess_on_match) or \
-           (not self.matched and self.reprocess_on_fail):
+        if ((self.matched and self.reprocess_on_match) or
+                (not self.matched and self.reprocess_on_fail)):
             reprocess.append([ProcessList(
                 items=[item],
                 work_over=self.work_over,
