@@ -72,7 +72,7 @@ def do_correction(input_model, barshadow_model=None, inverse=False, source_type=
         corrections.slits.append(correction)
 
         # Apply the correction by dividing into the science and uncertainty arrays:
-        #     var_poission and var_rnoise are divided by correction**2,
+        #     var_poisson and var_rnoise are divided by correction**2,
         #     because they're variance, while err is standard deviation
         if not inverse:
             slitlet.data /= correction.data
@@ -133,14 +133,26 @@ def _calc_correction(slitlet, barshadow_model, source_type):
             # Use this transformation to calculate x, y, and wavelength
             xslit, yslit, wavelength = det2slit(x, y)
 
+            # If the source position is off-center in the slit, renormalize the yslit
+            # values so that it appears as if the source is centered, which is the appropriate
+            # way to compute the shadow correction for extended/uniform sources (doesn't
+            # depend on source location).
+            if len(shutter_status) > 1:
+                middle = (len(shutter_status) - 1) / 2.0
+                src_loc = shutter_status.find('x')
+                if src_loc != -1 and float(src_loc) != middle:
+                    yslit -= np.nanmean(yslit)
+
             # The returned y values are scaled to where the slit height is 1
             # (i.e. a slit goes from -0.5 to 0.5).  The barshadow array is scaled
             # so that the separation between the slit centers is 1,
             # i.e. slit height + interslit bar
             yslit = yslit / SLITRATIO
 
-            # Convert the Y and wavelength to a pixel location in the  bar shadow array
-            index_of_fiducial = shutter_status.find('x')
+            # Convert the Y and wavelength to a pixel location in the  bar shadow array;
+            # the fiducial should always be at the center of the slitlet, regardless of
+            # where the source is centered.
+            index_of_fiducial = (len(shutter_status) - 1) / 2.0
 
             # The shutters go downwards, i.e. the first shutter in shutter_status corresponds to
             # the last in the shadow array.  So the center of the first shutter referred to in
@@ -275,7 +287,7 @@ def create_closed_open(shadow1x1):
 def create_closed_closed():
     """Create the two half shutters obtained from two closed shutters
     Uses 0.01 somewhat arbitrarily, although this case shouldn't occur
-    very often, only when there are 2 or more consecutuve closed shutters
+    very often, only when there are 2 or more consecutive closed shutters
     in a slitlet
 
     Parameters:
@@ -377,7 +389,7 @@ def add_first_half_shutter(shadow, shadow_element):
         The bar shadow array.
 
     shadow_element: nddata array
-        the shutter_elements['first'] array.  Should be 501 rows (Y) by 101 columns (wavelenth)
+        the shutter_elements['first'] array.  Should be 501 rows (Y) by 101 columns (wavelength)
 
     Returns:
 
@@ -437,7 +449,7 @@ def add_last_half_shutter(shadow, shadow_element, first_row):
     Returns:
 
     shadow: nddata array
-        The bar shadow array with the lastt half shutter inserted
+        The bar shadow array with the last half shutter inserted
 
     """
     #
