@@ -20,6 +20,7 @@ from astropy.nddata.bitmask import (
 from .skymatch import match
 from .skycube import SkyCube
 from ..skymatch.skystatistics import SkyStats
+from ..datamodels.dqflags import pixel
 
 __all__ = ['CubeSkyMatchStep']
 
@@ -30,6 +31,8 @@ class CubeSkyMatchStep(Step):
 
     """
 
+    class_alias = "cube_skymatch"
+
     spec = """
         # General sky matching parameters:
         bkg_degree = integer(min=0, default=1) # Degree of the polynomial for background fitting
@@ -52,7 +55,7 @@ class CubeSkyMatchStep(Step):
     def process(self, input1, input2):
         cube_models = datamodels.ModelContainer(input1)
         models2d = datamodels.ModelContainer(input2)
-        dqbits = interpret_bit_flags(self.dqbits)
+        dqbits = interpret_bit_flags(self.dqbits, flag_name_map=pixel)
 
         # set sky statistics:
         self._skystat = SkyStats(
@@ -274,9 +277,9 @@ class CubeSkyMatchStep(Step):
         y, x = np.indices((1024, 512))
 
         if channel in ['1', '3']:
-            return (x + 4, y)
+            return x + 4, y
         else:
-            return (x + 516, y)
+            return x + 516, y
 
 
 def _find_associated_2d_image(cube, models2d):
@@ -294,7 +297,7 @@ def _find_associated_2d_image(cube, models2d):
 
     for m in models2d:
         if m.meta.filename == fn:
-            return (m, ch)
+            return m, ch
 
     return None, None
 
