@@ -130,14 +130,15 @@ def ifu_extract1d(input_model, ref_dict, source_type, subtract_background,
     npixels_bkg_temp = np.where(npixels_bkg > 0., npixels_bkg, 1.)
 
     # Convert the sum to an average, for surface brightness.
+    # Remember variance requires dividing by N^2
     surf_bright = temp_flux / npixels_temp
-    sb_var_poisson = f_var_poisson / npixels_temp
-    sb_var_rnoise = f_var_rnoise / npixels_temp
-    sb_var_flat = f_var_flat / npixels_temp
-    background /= npixels_bkg_temp
-    b_var_poisson /= npixels_bkg_temp
-    b_var_rnoise /= npixels_bkg_temp
-    b_var_flat /= npixels_bkg_temp
+    sb_var_poisson = f_var_poisson / npixels_temp / npixels_temp
+    sb_var_rnoise = f_var_rnoise / npixels_temp / npixels_temp
+    sb_var_flat = f_var_flat / npixels_temp / npixels_temp
+    background = background / npixels_bkg_temp
+    b_var_poisson = b_var_poisson / npixels_bkg_temp / npixels_bkg_temp
+    b_var_rnoise = b_var_rnoise / npixels_bkg_temp / npixels_bkg_temp
+    b_var_flat = b_var_flat / npixels_bkg_temp / npixels_bkg_temp
 
     del npixels_temp
     del npixels_bkg_temp
@@ -176,6 +177,16 @@ def ifu_extract1d(input_model, ref_dict, source_type, subtract_background,
     berror = np.sqrt(b_var_poisson + b_var_rnoise + b_var_flat)
     spec_dtype = datamodels.SpecModel().spec_table.dtype
 
+    # If we only used the Poisson variance array as a vehicle to pass through
+    # non-differentiated errors, clear it again here so that only the total
+    # errors pass out into the 1d spectra files.
+    try:
+        input_model.var_poisson
+    except AttributeError:
+        f_var_poisson *= 0
+        sb_var_poisson *= 0
+        b_var_poisson *= 0
+
     otab = np.array(
         list(
             zip(wavelength, flux, error, f_var_poisson, f_var_rnoise, f_var_flat,
@@ -290,7 +301,8 @@ def get_extract_parameters(ref_dict, bkg_sigma_clip, slitname):
         extract_params['ref_file_type'] = FILE_TYPE_IMAGE
         foundit = False
         for im in ref_dict['ref_model'].images:
-            if (im.name is None or im.name == "ANY" or slitname == "ANY" or im.name == slitname):
+            if (im.name is None or im.name == "ANY" or slitname == "ANY" or
+                    im.name == slitname):
                 extract_params['ref_image'] = im
                 foundit = True
                 break
@@ -395,8 +407,8 @@ def extract_ifu(input_model, source_type, extract_params):
         var_rnoise = input_model.var_rnoise
         var_flat = input_model.var_flat
     except AttributeError:
-        log.info("Input model has no variance information. Creating zero-filled arrays.")
-        var_poisson = np.zeros_like(input_model.data)
+        log.info("Input model does not break out variance information. Passing only generalized errors.")
+        var_poisson = input_model.err * input_model.err
         var_rnoise = np.zeros_like(input_model.data)
         var_flat = np.zeros_like(input_model.data)
     weightmap = input_model.weightmap
@@ -563,8 +575,14 @@ def extract_ifu(input_model, source_type, extract_params):
                                          method=method, subpixels=subpixels)
 
         aperture_area = float(phot_table['aperture_sum'][0])
+        # if aperture_area = 0, then there is no valid data for this wavelength
+        # set the DQ flag to DO_NOT_USE
+        if aperture_area == 0:
+            dq[k] = dqflags.pixel['DO_NOT_USE']
 
-        if(aperture_area == 0 and aperture.area > 0):
+        # There is no valid data for this region. To prevent the code from
+        # crashing set aperture_area to a nonzero value. It will have the dq flag
+        if aperture_area == 0 and aperture.area > 0:
             aperture_area = aperture.area
 
         if subtract_background and annulus is not None:
@@ -573,7 +591,7 @@ def extract_ifu(input_model, source_type, extract_params):
                                              method=method, subpixels=subpixels)
             annulus_area = float(phot_table['aperture_sum'][0])
 
-            if(annulus_area == 0 and annulus.area > 0):
+            if annulus_area == 0 and annulus.area > 0:
                 annulus_area = annulus.area
 
             if annulus_area > 0.:
@@ -584,6 +602,7 @@ def extract_ifu(input_model, source_type, extract_params):
                 subtract_background_plane = False
 
         npixels[k] = aperture_area
+
         npixels_bkg[k] = 0.0
         if annulus is not None:
             npixels_bkg[k] = annulus_area
@@ -631,32 +650,34 @@ def extract_ifu(input_model, source_type, extract_params):
             # the edge data that is zero to define the statistics on clipping
             bkg_stat_data = bkg_data[temp_weightmap == 1]
 
-            bkg_mean, _, bkg_stddev = stats.sigma_clipped_stats(bkg_stat_data,
-                                                                sigma=bkg_sigma_clip, maxiters=5)
-            low = bkg_mean - bkg_sigma_clip * bkg_stddev
-            high = bkg_mean + bkg_sigma_clip * bkg_stddev
+            # If there are good data, work out the statistics
+            if len(bkg_stat_data) > 0:
+                bkg_mean, _, bkg_stddev = stats.sigma_clipped_stats(bkg_stat_data,
+                                                                    sigma=bkg_sigma_clip, maxiters=5)
+                low = bkg_mean - bkg_sigma_clip * bkg_stddev
+                high = bkg_mean + bkg_sigma_clip * bkg_stddev
 
-            # set up the mask to flag data that should not be used in aperture photometry
-            maskclip = np.logical_or(bkg_data < low, bkg_data > high)
+                # set up the mask to flag data that should not be used in aperture photometry
+                maskclip = np.logical_or(bkg_data < low, bkg_data > high)
 
-            bkg_table = aperture_photometry(bkg_data, aperture, mask=maskclip,
-                                            method=method, subpixels=subpixels)
-            background[k] = float(bkg_table['aperture_sum'][0])
-            phot_table = aperture_photometry(temp_weightmap, aperture, mask=maskclip,
-                                             method=method, subpixels=subpixels)
-            npixels_bkg[k] = float(phot_table['aperture_sum'][0])
-
-            var_poisson_table = aperture_photometry(var_poisson[k, :, :], aperture, mask=maskclip,
-                                                    method=method, subpixels=subpixels)
-            b_var_poisson[k] = float(var_poisson_table['aperture_sum'][0])
-
-            var_rnoise_table = aperture_photometry(var_rnoise[k, :, :], aperture, mask=maskclip,
-                                                   method=method, subpixels=subpixels)
-            b_var_rnoise[k] = float(var_rnoise_table['aperture_sum'][0])
-
-            var_flat_table = aperture_photometry(var_flat[k, :, :], aperture, mask=maskclip,
+                bkg_table = aperture_photometry(bkg_data, aperture, mask=maskclip,
+                                                method=method, subpixels=subpixels)
+                background[k] = float(bkg_table['aperture_sum'][0])
+                phot_table = aperture_photometry(temp_weightmap, aperture, mask=maskclip,
                                                  method=method, subpixels=subpixels)
-            b_var_flat[k] = float(var_flat_table['aperture_sum'][0])
+                npixels_bkg[k] = float(phot_table['aperture_sum'][0])
+
+                var_poisson_table = aperture_photometry(var_poisson[k, :, :], aperture, mask=maskclip,
+                                                        method=method, subpixels=subpixels)
+                b_var_poisson[k] = float(var_poisson_table['aperture_sum'][0])
+
+                var_rnoise_table = aperture_photometry(var_rnoise[k, :, :], aperture, mask=maskclip,
+                                                       method=method, subpixels=subpixels)
+                b_var_rnoise[k] = float(var_rnoise_table['aperture_sum'][0])
+
+                var_flat_table = aperture_photometry(var_flat[k, :, :], aperture, mask=maskclip,
+                                                     method=method, subpixels=subpixels)
+                b_var_flat[k] = float(var_flat_table['aperture_sum'][0])
 
         del temp_weightmap
         # done looping over wavelength bins
@@ -872,8 +893,8 @@ def image_extract_ifu(input_model, source_type, extract_params):
         var_rnoise = input_model.var_rnoise
         var_flat = input_model.var_flat
     except AttributeError:
-        log.info("Input model has no variance information. Creating zero-filled arrays.")
-        var_poisson = np.zeros_like(input_model.data)
+        log.info("Input model does not break out variance information. Passing only generalized errors.")
+        var_poisson = input_model.err * input_model.err
         var_rnoise = np.zeros_like(input_model.data)
         var_flat = np.zeros_like(input_model.data)
     # The axes are (z, y, x) in the sense that (x, y) are the ordinary
@@ -1067,7 +1088,7 @@ def get_coordinates(input_model, x0, y0):
         (ra, dec) = (0., 0.)
         wavelength = np.arange(1, nelem + 1, dtype=np.float64)
 
-    return (ra, dec, wavelength)
+    return ra, dec, wavelength
 
 
 def nans_in_wavelength(wavelength, dq):
@@ -1098,14 +1119,14 @@ def nans_in_wavelength(wavelength, dq):
     slc = slice(nelem)
     if nelem == 0:
         log.warning("Output arrays are empty!")
-        return (wavelength, dq, slice(nelem))
+        return wavelength, dq, slice(nelem)
 
     nan_mask = np.isnan(wavelength)
     n_nan = nan_mask.sum(dtype=np.intp)
     if n_nan == nelem:
         log.warning("Wavelength array is all NaN!")
         dq = np.bitwise_or(dq[:], dqflags.pixel['DO_NOT_USE'])
-        return (wavelength, dq, slice(0))
+        return wavelength, dq, slice(0)
 
     if n_nan > 0:
         log.warning("%d NaNs in wavelength array.", n_nan)
@@ -1162,7 +1183,7 @@ def separate_target_and_background(ref):
     else:
         mask_bkg = None
 
-    return (mask_target, mask_bkg)
+    return mask_target, mask_bkg
 
 
 def im_centroid(data, mask_target):
@@ -1198,7 +1219,7 @@ def im_centroid(data, mask_target):
         shape = data_2d.shape
         y0 = shape[0] / 2.
         x0 = shape[0] / 2.
-        return (y0, x0)
+        return y0, x0
 
     x_profile = data_2d.sum(axis=0, dtype=np.float64)
     x = np.arange(data_2d.shape[1], dtype=np.float64)
@@ -1210,7 +1231,7 @@ def im_centroid(data, mask_target):
     s_y = (y_profile * y).sum()
     y0 = s_y / y_profile.sum()
 
-    return (y0, x0)
+    return y0, x0
 
 
 def shift_ref_image(mask, delta_y, delta_x, fill=0):
