@@ -10,7 +10,7 @@ import numpy as np
 
 from .. stpipe import Step
 from .. import datamodels
-from .. wiimatch.match import match_lsq
+from wiimatch.match import match_lsq
 from astropy.stats import sigma_clipped_stats as sigclip
 
 __all__ = ['MRSIMatchStep', 'apply_background_2d']
@@ -22,6 +22,8 @@ class MRSIMatchStep(Step):
 
     """
 
+    class_alias = "mrs_imatch"
+
     spec = """
         # General sky matching parameters:
         bkg_degree = integer(min=0, default=1) # Degree of the polynomial for background fitting
@@ -246,9 +248,9 @@ def _get_2d_pixgrid(model2d, channel):
     y, x = np.indices((1024, 512))
 
     if channel in ['1', '4']:
-        return (x + 4, y)
+        return x + 4, y
     else:
-        return (x + 516, y)
+        return x + 516, y
 
 
 def _match_models(models, channel, degree, center=None, center_cs='image'):
@@ -259,7 +261,7 @@ def _match_models(models, channel, degree, center=None, center_cs='image'):
     cbs.channel = str(channel)
     cbs.band = 'ALL'
     cbs.single = True
-    cbs.weighting = 'EMSM'
+    cbs.weighting = 'drizzle'
     cube_models = cbs.process(models)
     if len(cube_models) != len(models):
         raise RuntimeError("The number of generated cube models does not "
