@@ -41,6 +41,8 @@ class OutlierDetectionStep(Step):
 
     """
 
+    class_alias = "outlier_detection"
+
     # The members of spec needs to be a super-set of all parameters needed
     # by the various versions of the outlier_detection algorithms, and each
     # version will pick and choose what they need while ignoring the rest.
@@ -62,11 +64,14 @@ class OutlierDetectionStep(Step):
         scale_detection = boolean(default=False)
         search_output_file = boolean(default=False)
         allowed_memory = float(default=None)  # Fraction of memory to use for the combined image.
+        in_memory = boolean(default=False)
     """
 
     def process(self, input_data):
         """Perform outlier detection processing on input data."""
-        with datamodels.open(input_data) as input_models:
+
+        # interpret how memory should be used
+        with datamodels.open(input_data, save_open=self.in_memory) as input_models:
             self.input_models = input_models
             if not isinstance(self.input_models, datamodels.ModelContainer):
                 self.input_container = False
@@ -104,6 +109,7 @@ class OutlierDetectionStep(Step):
                 'scale': self.scale,
                 'backg': self.backg,
                 'allowed_memory': self.allowed_memory,
+                'in_memory': self.in_memory,
                 'save_intermediate_results': self.save_intermediate_results,
                 'resample_data': self.resample_data,
                 'good_bits': self.good_bits,
@@ -160,10 +166,8 @@ class OutlierDetectionStep(Step):
             if self.input_container:
                 for model in self.input_models:
                     model.meta.cal_step.outlier_detection = state
-                    model.meta.filetype = 'cosmic-ray flagged'
             else:
                 self.input_models.meta.cal_step.outlier_detection = state
-                self.input_models.meta.filetype = 'cosmic-ray flagged'
 
             return self.input_models
 
