@@ -1,9 +1,11 @@
 import numpy as np
 import numpy.testing as npt
 
-from jwst.ramp_fitting.ramp_fit import ramp_fit
+from stcal.ramp_fitting.ramp_fit import ramp_fit
+
 from jwst.datamodels import RampModel
 from jwst.datamodels import GainModel, ReadnoiseModel
+from jwst.datamodels import dqflags
 
 #
 # The first 12 tests are for a single ramp in a single integration. The ramps
@@ -17,6 +19,12 @@ from jwst.datamodels import GainModel, ReadnoiseModel
 #  their SCI values (these are mostly for my reference).
 #
 
+test_dq_flags = dqflags.pixel
+
+DO_NOT_USE = test_dq_flags["DO_NOT_USE"]
+JUMP_DET = test_dq_flags["JUMP_DET"]
+SATURATED = test_dq_flags["SATURATED"]
+
 
 def test_pix_0():
     """
@@ -25,16 +33,17 @@ def test_pix_0():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 55.,
-                                            65., 75., 94., 95., 105.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 55., 65., 75., 94., 95., 105.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     # [data, dq, err, var_p, var_r]
@@ -60,16 +69,17 @@ def test_pix_1():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 55., 65., 75.,
-                                            94., 95., 105.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 55., 65., 75., 94., 95., 105.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 4, 4, 0, 2, 2, 2, 2, 2, 2])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1.8999999, 6, 1.05057204, 0.03454545, 1.0691562]
@@ -90,16 +100,17 @@ def test_pix_2():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 55., 65., 75., 94.,
-                                            95., 105.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 55., 65., 75., 94., 95., 105.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 0, 0, 4, 0, 4, 0, 4, 2, 2])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [0.84833729, 6, 0.42747884, 0.00454545, 0.1781927]
@@ -127,16 +138,17 @@ def test_pix_3():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 55., 65., 75.,
-                                            94., 95., 105.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 55., 65., 75., 94., 95., 105.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 0, 0, 0, 0, 0, 0, 0, 4, 0])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1.0746869, 4, 0.12186482, 0.00227273, 0.01257831]
@@ -164,16 +176,17 @@ def test_pix_4():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 1055., 1065., 1075.,
-                                            2594., 2595., 2605.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 1055., 1065., 1075., 2594., 2595., 2605.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 2, 2, 2, 2, 2, 2, 2, 2, 2])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1.5, 2, 1.047105, 0.02727273, 1.0691562]
@@ -194,16 +207,17 @@ def test_pix_5():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 2055., 2065., 2075.,
-                                            2094., 2095., 2105.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 2055., 2065., 2075., 2094., 2095., 2105.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 0, 0, 0, 4, 0, 0, 0, 0, 0])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1.076075, 4, 0.16134359, 0.00227273, 0.02375903]
@@ -231,16 +245,17 @@ def test_pix_6():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 55., 65., 375.,
-                                            394., 395., 405.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 55., 65., 375., 394., 395., 405.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 0, 4, 4, 0, 0, 0, 0, 0, 0])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [6.092052, 4, 0.14613187, 0.0025974, 0.01875712]
@@ -267,16 +282,17 @@ def test_pix_7():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 55., 65., 75.,
-                                            94., 195., 205.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 55., 65., 75., 94., 195., 205.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 0, 0, 0, 0, 0, 0, 0, 4, 4])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1.0757396, 4, 0.12379601, 0.0025974, 0.01272805]
@@ -297,16 +313,17 @@ def test_pix_8():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 55., 65., 75.,
-                                            94., 95., 105.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 55., 65., 75., 94., 95., 105.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 4, 0, 0, 0, 0, 0, 2, 2, 2])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1.0101178, 6, 0.1848883, 0.00363636, 0.03054732]
@@ -328,16 +345,17 @@ def test_pix_9():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 55., 65., 75.,
-                                            94., 95., 105.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 55., 65., 75., 94., 95., 105.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 0, 4, 4, 0, 0, 0, 0, 4, 0])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [0.9999994, 4, 0.22721863, 0.0030303, 0.048598]
@@ -366,16 +384,17 @@ def test_pix_10():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 55., 65., 75.,
-                                            94., 95., 105.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 55., 65., 75., 94., 95., 105.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 0, 4, 0, 0, 4, 0, 0, 0, 0])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1., 4, 0.21298744, 0.0025974, 0.04276625]
@@ -402,16 +421,17 @@ def test_pix_11():
     """
 
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([15., 25., 35., 54., 55., 65., 75.,
-                                            94., 95., 105.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [15., 25., 35., 54., 55., 65., 75., 94., 95., 105.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([0, 0, 2, 2, 2, 2, 2, 2, 2, 2])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1., 2, 1.042755, 0.01818182, 1.0691562]
@@ -434,8 +454,8 @@ def test_pix_12():
     ngroups = 2
     nints = 1
     ncols = 2
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
     ramp_model.data[0, :, 0, 0] = np.array([15., 59025.], dtype=np.float32)
@@ -444,8 +464,10 @@ def test_pix_12():
     ramp_model.groupdq[0, :, 0, 1] = np.array([2, 2])
 
     # call ramp_fit
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
+
     # Set truth values for PRIMARY results for pixel 1:
     # slope, dq, err, var_p, var_r
     # slope = group1 / deltatime = 15 / 10 = 1.5
@@ -484,16 +506,17 @@ def test_miri_0():
     GROUPDQ is: [1, 0, 0, 0, 0, 0, 0, 0, 0, 1]
     """
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([8888., 25., 35., 54., 55.,
-                                            65., 75., 94., 95., 888.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [8888., 25., 35., 54., 55., 65., 75., 94., 95., 888.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([1, 0, 0, 0, 0, 0, 0, 0, 0, 1])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1.025854, 0, 0.12379601, 0.0025974, 0.01272805]
@@ -514,16 +537,17 @@ def test_miri_1():
     GROUPDQ is: [5, 0, 0, 0, 0, 0, 0, 0, 0, 1]
     """
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([7777., 125., 135., 154., 165., 175.,
-                                            185., 204., 205., 777.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [7777., 125., 135., 154., 165., 175., 185., 204., 205., 777.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([5, 0, 0, 0, 0, 0, 0, 0, 0, 1])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1.1996487, 0, 0.12379601, 0.0025974, 0.01272805]
@@ -544,16 +568,17 @@ def test_miri_2():
     GROUPDQ is: [5, 0, 0, 0, 0, 0, 0, 0, 0, 5]
     """
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([4444., 25., 35., 54., 55., 65., 75.,
-                                            94., 95., 444.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [4444., 25., 35., 54., 55., 65., 75., 94., 95., 444.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([5, 0, 0, 0, 0, 0, 0, 0, 0, 5])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1.025854, 0, 0.12379601, 0.0025974, 0.01272805]
@@ -574,16 +599,17 @@ def test_miri_3():
     GROUPDQ is: [1, 0, 0, 0, 0, 0, 0, 0, 0, 5]
     """
     ngroups, nints, nrows, ncols, deltatime, gain, readnoise = set_scalars()
-    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(ngroups,
-                                                                                  nints, nrows, ncols, deltatime, gain, readnoise)
+    ramp_model, rnoise_model, gain_model, pixdq, groupdq, err = create_mod_arrays(
+        ngroups, nints, nrows, ncols, deltatime, gain, readnoise)
 
     # Populate pixel-specific SCI and GROUPDQ arrays
-    ramp_model.data[0, :, 0, 0] = np.array([6666., 25., 35., 54., 55., 65., 75.,
-                                            94., 95., 666.], dtype=np.float32)
+    ramp_model.data[0, :, 0, 0] = np.array(
+        [6666., 25., 35., 54., 55., 65., 75., 94., 95., 666.], dtype=np.float32)
     ramp_model.groupdq[0, :, 0, 0] = np.array([1, 0, 0, 0, 0, 0, 0, 0, 0, 5])
 
-    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(ramp_model, 1024 * 300., True,
-                                                      rnoise_model, gain_model, 'OLS', 'optimal', 'none')
+    new_mod, int_mod, opt_mod, gls_opt_mod = ramp_fit(
+        ramp_model, 1024 * 300., True, rnoise_model, gain_model,
+        'OLS', 'optimal', 'none', test_dq_flags)
 
     # Set truth values for PRIMARY results:
     p_true = [1.025854, 0, 0.12379601, 0.0025974, 0.01272805]
@@ -596,37 +622,41 @@ def test_miri_3():
     assert_opt(o_true, opt_mod, 0)
 
 
-def assert_pri(p_true, new_mod, pix):
+def assert_pri(p_true, new_info, pix):
     """
     Compare true and fit values of primary output for extensions
-    SCI, DQ, ERR, VAR_POISSSON, VAR_RNOISE.
+    SCI, DQ, ERR, VAR_POISSON, VAR_RNOISE.
     """
 
-    npt.assert_allclose(new_mod.data[0, pix], p_true[0], atol=2E-5, rtol=2e-5)
-    npt.assert_allclose(new_mod.dq[0, pix], p_true[1], atol=1E-1)
-    npt.assert_allclose(new_mod.err[0, pix], p_true[2], atol=2E-5, rtol=2e-5)
-    npt.assert_allclose(new_mod.var_poisson[0, pix], p_true[3], atol=2E-5, rtol=2e-5)
-    npt.assert_allclose(new_mod.var_rnoise[0, pix], p_true[4], atol=2E-5, rtol=2e-5)
+    data, dq, var_poisson, var_rnoise, err = new_info
+
+    npt.assert_allclose(data[0, pix], p_true[0], atol=2E-5, rtol=2e-5)
+    npt.assert_allclose(dq[0, pix], p_true[1], atol=1E-1)
+    npt.assert_allclose(err[0, pix], p_true[2], atol=2E-5, rtol=2e-5)
+    npt.assert_allclose(var_poisson[0, pix], p_true[3], atol=2E-5, rtol=2e-5)
+    npt.assert_allclose(var_rnoise[0, pix], p_true[4], atol=2E-5, rtol=2e-5)
 
     return None
 
 
-def assert_opt(o_true, opt_mod, pix):
+def assert_opt(o_true, opt_info, pix):
     """
     Compare true and fit values of optional output for extensions SLOPE,
-    SIGSLOPE, VAR_POISSSON, VAR_RNOISE, YINT, SIGYINT, PEDESTAL, and WEIGHTS.
+    SIGSLOPE, VAR_POISSON, VAR_RNOISE, YINT, SIGYINT, PEDESTAL, and WEIGHTS.
     Selecting the particular (and only) ramp in the optional output, which is
     [0,:,0,0]
     """
+    (slope, sigslope, var_poisson, var_rnoise,
+        yint, sigyint, pedestal, weights, crmag) = opt_info
 
-    opt_slope = opt_mod.slope[0, :, 0, pix]
-    opt_sigslope = opt_mod.sigslope[0, :, 0, pix]
-    opt_var_poisson = opt_mod.var_poisson[0, :, 0, pix]
-    opt_var_rnoise = opt_mod.var_rnoise[0, :, 0, pix]
-    opt_yint = opt_mod.yint[0, :, 0, pix]
-    opt_sigyint = opt_mod.sigyint[0, :, 0, pix]
-    opt_pedestal = opt_mod.pedestal[:, 0, pix]
-    opt_weights = opt_mod.weights[0, :, 0, pix]
+    opt_slope = slope[0, :, 0, pix]
+    opt_sigslope = sigslope[0, :, 0, pix]
+    opt_var_poisson = var_poisson[0, :, 0, pix]
+    opt_var_rnoise = var_rnoise[0, :, 0, pix]
+    opt_yint = yint[0, :, 0, pix]
+    opt_sigyint = sigyint[0, :, 0, pix]
+    opt_pedestal = pedestal[:, 0, pix]
+    opt_weights = weights[0, :, 0, pix]
 
     npt.assert_allclose(opt_slope, o_true[0], atol=2E-5, rtol=2e-5)
     npt.assert_allclose(opt_sigslope, o_true[1], atol=2E-5, rtol=2e-5)
@@ -693,6 +723,8 @@ def create_mod_arrays(ngroups, nints, nrows, ncols, deltatime, gain, readnoise):
     ramp_model.meta.exposure.groupgap = 0
     ramp_model.meta.exposure.drop_frames1 = 0
 
+    return ramp_model, read_noise, gain, pixdq, gdq, err
+
     # Create and populate gain model
     gain_model = GainModel(data=gain)
     gain_model.meta.instrument.name = 'MIRI'
