@@ -8,14 +8,10 @@ from jwst.stpipe import Step
 def run_pipeline(jail, rtdata_module):
     """Run calwebb_coron3 on coronographic data."""
     rtdata = rtdata_module
-    psfmask = rtdata.get_data("nircam/coron/jwst_nircam_psfmask_somb.fits")
-    rtdata.get_asn("nircam/coron/jw99999-a3001_20170327t121212_coron3_001_asn.json")
+    rtdata.get_asn("nircam/coron/jw01386-c1020_20220909t073458_coron3_002a_asn.json")
 
     # Run the calwebb_coron3 pipeline on the association
-    args = [
-        "calwebb_coron3", rtdata.input,
-        f"--steps.align_refs.override_psfmask={psfmask}"
-    ]
+    args = ["calwebb_coron3", rtdata.input]
     Step.from_cmdline(args)
 
     return rtdata
@@ -23,12 +19,12 @@ def run_pipeline(jail, rtdata_module):
 
 @pytest.mark.bigdata
 @pytest.mark.parametrize("suffix", ["psfalign", "psfsub", "crfints"])
-@pytest.mark.parametrize("exposure", ["00001", "00002"])
-def test_nircam_coron3_sci_exp(run_pipeline, suffix, exposure, fitsdiff_default_kwargs):
+@pytest.mark.parametrize("obs", ["002", "003"])
+def test_nircam_coron3_sci_exp(run_pipeline, suffix, obs, fitsdiff_default_kwargs):
     """Check intermediate results of calwebb_coron3"""
     rtdata = run_pipeline
 
-    output = "jw9999947001_02102_" + exposure + "_nrcb3_a3001_" + suffix + ".fits"
+    output = "jw01386" + obs + "001_03108_00001_nrcalong_c1020_" + suffix + ".fits"
     rtdata.output = output
     rtdata.get_truth("truth/test_nircam_coron3/" + output)
 
@@ -39,12 +35,12 @@ def test_nircam_coron3_sci_exp(run_pipeline, suffix, exposure, fitsdiff_default_
 
 @pytest.mark.bigdata
 @pytest.mark.parametrize("suffix", ["crfints"])
-@pytest.mark.parametrize("exposure", ["00003", "00004", "00005"])
+@pytest.mark.parametrize("exposure", ["00001", "00002", "00003", "00004"])
 def test_nircam_coron3_psf_exp(run_pipeline, suffix, exposure, fitsdiff_default_kwargs):
     """Check intermediate results of calwebb_coron3"""
     rtdata = run_pipeline
 
-    output = "jw9999947001_02102_" + exposure + "_nrcb3_a3001_" + suffix + ".fits"
+    output = "jw01386001001_0310a_" + exposure + "_nrcalong_c1020_" + suffix + ".fits"
     rtdata.output = output
     rtdata.get_truth("truth/test_nircam_coron3/" + output)
 
@@ -59,7 +55,7 @@ def test_nircam_coron3_product(run_pipeline, suffix, fitsdiff_default_kwargs):
     """Check final products of calwebb_coron3"""
     rtdata = run_pipeline
 
-    output = "jw99999-a3001_t1_nircam_f140m-maskbar_" + suffix + ".fits"
+    output = "jw01386-c1020_t001_nircam_f410m-maskrnd-sub320a335r_mini_" + suffix + ".fits"
     rtdata.output = output
     rtdata.get_truth("truth/test_nircam_coron3/" + output)
 
