@@ -4,17 +4,20 @@ import numpy.polynomial.polynomial as poly
 
 from scipy.interpolate import pchip
 from astropy.timeseries import LombScargle
-from BayesicFitting import SplinesModel
-from BayesicFitting import Fitter
 from BayesicFitting import SineModel
 from BayesicFitting import LevenbergMarquardtFitter
 from BayesicFitting import RobustShell
+from BayesicFitting import ConstantModel
+from BayesicFitting import Fitter
 
-from numpy.linalg.linalg import LinAlgError
+from astropy.modeling.models import Spline1D
+from astropy.modeling.fitting import SplineExactKnotsFitter
+
+from .fitter import ChiSqOutlierRejectionFitter
 
 import logging
 log = logging.getLogger(__name__)
-log.setLevel(logging.DEBUG)
+log.setLevel(logging.INFO)
 #
 
 # hard coded parameters, have been selected based on testing but can be changed
@@ -39,7 +42,7 @@ def slice_info(slice_map, c):
                                               & (slice_inventory < 100 * (c + 1)))]
 
     log.info("Number of slices in band {} ".format(slices_in_band.shape[0]))
-    slice_x_ranges = np.zeros((slices_in_band.shape[0], 3),dtype=int)
+    slice_x_ranges = np.zeros((slices_in_band.shape[0], 3), dtype=int)
     all_slice_masks = np.zeros((slices_in_band.shape[0], slice_map.shape[0], slice_map.shape[1]))
     for n, s in enumerate(slices_in_band):
         # create a mask of the slice
@@ -55,18 +58,18 @@ def slice_info(slice_map, c):
         collapsed_slice = np.sum(slice, axis=0)
         indices = np.where(collapsed_slice[:-1] != collapsed_slice[1:])[0]
         slice_x_ranges[n, 0], slice_x_ranges[n, 1], slice_x_ranges[n, 2] = int(s), \
-            int(np.amin(indices)),int(np.amax(indices) + 1)
+            int(np.amin(indices)), int(np.amax(indices) + 1)
 
         log.debug("For slice {} x ranges of slices region {}, {}".
-                  format(slice_x_ranges[n,0],slice_x_ranges[n,1],slice_x_ranges[n,2]))
+                  format(slice_x_ranges[n, 0], slice_x_ranges[n, 1], slice_x_ranges[n, 2]))
 
     log.debug("Min and max x pixel values of all slices in channel {} {}".
-              format(np.amin(slice_x_ranges[:,1]), np.amax(slice_x_ranges[:,2])))
+              format(np.amin(slice_x_ranges[:, 1]), np.amax(slice_x_ranges[:, 2])))
 
     xrange_channel = np.zeros(2)
-    xrange_channel[0] = np.amin(slice_x_ranges[:,1])
-    xrange_channel[1] = np.amax(slice_x_ranges[:,2])
-    result = (slices_in_band, xrange_channel, slice_x_ranges,all_slice_masks)
+    xrange_channel[0] = np.amin(slice_x_ranges[:, 1])
+    xrange_channel[1] = np.amax(slice_x_ranges[:, 2])
+    result = (slices_in_band, xrange_channel, slice_x_ranges, all_slice_masks)
     return result
 
 
@@ -95,14 +98,14 @@ def fill_wavenumbers(wnums):
     idx = np.isfinite(wnums)
 
     # fit the on-slice wavenumbers
-    coeffs = poly.polyfit(np.arange(wnums.shape[0])[idx], wnums[idx], 3)
-    wnums_filled = poly.polyval(np.arange(wnums.shape[0]), coeffs)
+    coefs = poly.polyfit(np.arange(wnums.shape[0])[idx], wnums[idx], 3)
+    wnums_filled = poly.polyval(np.arange(wnums.shape[0]), coefs)
 
     # keep the original wnums for the on-slice pixels
     wnums_filled[idx] = wnums[idx]
 
     # clean
-    del idx, coeffs
+    del idx, coefs
 
     return wnums_filled
 
@@ -169,10 +172,12 @@ def fit_envelope(wavenum, signal):
     u_y = [signal[0]]
 
     for k in np.arange(1, len(signal) - 1):
-        if (np.sign(signal[k] - signal[k - 1]) == -1) and ((np.sign(signal[k] - signal[k + 1])) == -1):
+        if ((np.sign(signal[k] - signal[k - 1]) == -1) and
+                ((np.sign(signal[k] - signal[k + 1])) == -1)):
             l_x.append(wavenum[k])
             l_y.append(signal[k])
-        if (np.sign(signal[k] - signal[k - 1]) == 1) and ((np.sign(signal[k] - signal[k + 1])) == 1):
+        if ((np.sign(signal[k] - signal[k - 1]) == 1) and
+                ((np.sign(signal[k] - signal[k + 1])) == 1)):
             u_x.append(wavenum[k])
             u_y.append(signal[k])
 
@@ -189,8 +194,11 @@ def fit_envelope(wavenum, signal):
     return pcl(wavenum), l_x, l_y, pcu(wavenum), u_x, u_y
 
 
-def find_lines(signal, max_amp):
+def find_lines_resfringe(signal, max_amp):
     """
+    *** Replaced with find_lines below. This version does not include some of the
+    feature finding functionality***
+
     Take signal and max amp array, determine location of spectral
     features with amplitudes greater than max amp
 
@@ -210,25 +218,121 @@ def find_lines(signal, max_amp):
     u_y, u_x, l_y, l_x = [], [], [], []
 
     for x in r_x:
-        if (np.sign(signal_check[x] - signal_check[x - 1]) == 1) and (np.sign(signal_check[x] - signal_check[x + 1]) == 1):
+        if ((np.sign(signal_check[x] - signal_check[x - 1]) == 1) and
+                (np.sign(signal_check[x] - signal_check[x + 1]) == 1)):
             u_y.append(signal_check[x])
             u_x.append(x)
 
-        if (np.sign(signal_check[x] - signal_check[x - 1]) == -1) and (np.sign(signal_check[x] - signal_check[x + 1]) == -1):
+        if ((np.sign(signal_check[x] - signal_check[x - 1]) == -1) and
+                (np.sign(signal_check[x] - signal_check[x + 1]) == -1)):
             l_y.append(signal[x])
             l_x.append(x)
 
     log.debug("find_lines: Found {} peaks   {} troughs".format(len(u_x), len(l_x)))
-    weights_factors[signal_check > np.amax(max_amp)] = 0
+    weights_factors[signal_check > max_amp] = 0
+    return weights_factors
+
+
+def find_lines(signal, max_amp):
+    """
+    Take signal and max amp array, determine location of spectral
+    features with amplitudes greater than max amp
+
+    :param signal:
+    :param max_amp:
+    :return:
+    """
+
+    r_x = np.arange(signal.shape[0] - 1)
+
+    # setup the output arrays
+    signal_check = signal.copy()
+    weights_factors = np.ones(signal.shape[0])
+
+    # Detect peaks
+    u_y, u_x, l_y, l_x = [], [], [], []
+
+    for x in r_x:
+        if (np.sign(signal_check[x] - signal_check[x - 1]) == 1) and \
+                (np.sign(signal_check[x] - signal_check[x + 1]) == 1):
+            u_y.append(signal_check[x])
+            u_x.append(x)
+
+        if (np.sign(signal_check[x] - signal_check[x - 1]) == -1) and \
+                (np.sign(signal_check[x] - signal_check[x + 1]) == -1):
+            l_y.append(signal[x])
+            l_x.append(x)
+
+    for n, amp in enumerate(u_y):
+        max_amp_val = max_amp[u_x[n]]
+        log.debug("find_lines: check if peak above max amp")
+        if amp > max_amp_val:
+
+            # peak in x
+            # log.debug("find_lines: flagging neighbours")
+            xpeaks = [u_x[n] - 1, u_x[n], u_x[n] + 1]
+
+            # log.debug("find_lines:  find neareast troughs")
+            # find nearest troughs
+
+            for xp in xpeaks:
+                log.debug("find_lines:  checking ind {}".format(xp))
+
+                try:
+
+                    x1 = l_x[np.argsort(np.abs(l_x - xp))[0]]
+
+                    try:
+                        x2 = l_x[np.argsort(np.abs(l_x - xp))[1]]
+
+                        if x1 < x2:
+                            xlow = x1
+                            xhigh = x2
+                        if x1 > x2:
+                            xhigh = x1
+                            xlow = x2
+
+                    except IndexError:
+                        # raised if x1 is at the edge
+                        xlow = x1
+                        xhigh = x1
+
+                    # set the weights to 0 and signal 1
+                    log.debug("find_lines: setting weights between troughs to 0")
+                    signal_check[xlow:xhigh] = 0
+                    weights_factors[xlow:xhigh] = 0
+
+                except IndexError:
+                    pass
+
+    log.debug("find_lines: Found {} peaks   {} troughs".format(len(u_x), len(l_x)))
+    weights_factors[signal_check > max_amp * 2] = 0  # catch any remaining
+    # weights_factors[signal_check > np.amax(max_amp)] = 0
+
     return weights_factors
 
 
 def check_res_fringes(res_fringe_fit, max_amp):
     """
-    Check for regions where res fringe fit runs away, set to 0
+    Check for regions where res fringe fit runs away (greater than max amp),
+    set the beat where this happens to 0 to avoid  making the fringes worse
+
+    :Parameters:
+
+    res_fringe_fit:  numpy array, required
+        the residual fringe fit
+
+    max_amp:  numpy array, required
+        the maximum amplitude array
+
+    :Returns:
+
+    res_fringe_fit: numpy array
+        the residual fringe fit with exploding fit regions removed
+
+    flats: numpy array
+        flags where the fit was rejected
 
-    :param res_fringes:
-    :return:
     """
 
     flags = np.zeros(res_fringe_fit.shape[0])
@@ -247,11 +351,11 @@ def check_res_fringes(res_fringe_fit, max_amp):
     log.debug("check_res_fringes: found {} nodes".format(len(node_ind)))
 
     # find where res_fringes goes above max_amp
-    runaway_rfc = np.argwhere((np.abs(lenv_fit) + np.abs(uenv_fit)) > max_amp * 2)
+    runaway_rfc = np.argwhere((np.abs(lenv_fit) + np.abs(uenv_fit)) > (max_amp * 2))
 
     # check which signal env the blow ups are located in and set to 1, and set a flag array
     if len(runaway_rfc) > 0:
-        log.debug("check_res_fringes: {} data points exceed threshold".format(len(runaway_rfc)))
+        log.debug("check_res_fringes: {} data points exceed threhold".format(len(runaway_rfc)))
         log.debug("check_res_fringes: resetting fits to related beats")
         for i in runaway_rfc:
             # find where the index is compared to the nodes
@@ -284,107 +388,8 @@ def interp_helper(mask):
     return mask < 1e-05, lambda z: z.nonzero()[0]
 
 
-def make_knots(flux, nknots=20, weights=None):
-    """Defines knot positions for piecewise models. This simply splits the array into sections. It does
-    NOT take into account the shape of the data.
-
-    :Parameters:
-
-    flux: numpy array, required
-        the flux array or any array of the same dimension
-
-    nknots: int, optional, default=20
-        the number of knots to create (excluding 0 and 1023)
-
-    weights: numpy array, optional, default=None
-        optionally supply a weights array. This will be used to add knots at the edge of bad pixels or features
-
-    :Returns:
-
-    knot_idx, numpy array
-        the indices of the knots
-
-    """
-    log.debug("make_knots: creating {} knots on flux array".format(nknots))
-
-    # create an array of indices
-    npoints = flux.shape[0]
-
-    # kstep is the number of points / number of knots
-    knot_step = npoints // nknots  # + 1
-
-    # define an initial knot index array
-    init_knot_idx = np.zeros(nknots + 1)
-    for n in range(nknots):
-        init_knot_idx[n] = n * knot_step
-    init_knot_idx[-1] = npoints - 1
-
-    # get difference between the indices
-    knot_split = np.ediff1d(init_knot_idx)
-
-    # the last diff will always be different than the others
-    last_split = knot_split[-1]
-
-    # create new knot array with knots at 0 and 1023, then inital and final splits = last_split/2
-    knot_idx = np.ones(nknots + 2)
-    for n in range(nknots):
-        knot_idx[n + 1] = (n * knot_step) + (last_split / 2)
-    knot_idx[0] = 0
-    knot_idx[-1] = npoints - 1
-
-    # if the weights array is supplied, determine the edges of good data and set knots there
-    if weights is not None:
-
-        log.debug("make_knots: adding knots at edges of bad pixels in weights array")
-
-        # if there are bad pixels in the flux array with flux~0,
-        # add these to weights array if not already there
-        weights *= (flux > 1e-03).astype(int)
-
-        # use a two-point difference method
-        weights_diff = np.ediff1d(weights)
-
-        # set edges where diff should be almost equal to the largest of the two datapoints used
-        # iterate over the diffs and compare to the datapoints
-        edges_idx_list = []
-        for n, wd in enumerate(weights_diff):
-            # get the data points used for the diff
-            datapoints = np.array([weights[n], weights[n + 1]])
-
-            # get the value and index of the larges
-            largest = np.amax(datapoints)
-            largest_idx = np.argmax(datapoints)
-
-            # we don't need knots in the bad pixels so ignore these
-            if largest > 1e-03:
-
-                # check if the absolute values are almost equal
-                if math.isclose(largest, np.abs(wd), rel_tol=1e-01):
-                    # if so, set the index and adjust depending on whether the
-                    # first or second datapoint is the largest
-                    idx = n + largest_idx
-
-                    # check if this is right next to another index already defined
-                    # causes problems in fitting, minimal difference
-                    if (idx - 1 in knot_idx) | (idx + 1 in knot_idx):
-                        pass
-                    else:
-                        # append to the index list
-                        edges_idx_list.append(idx)
-
-                else:
-                    pass
-
-        # convert the list to array, add to the knot_idx array, remove duplicates and sort
-        edges_idx = np.asarray(edges_idx_list)
-        knot_idx = np.sort(np.concatenate((knot_idx, edges_idx), axis=0), axis=0)
-        knot_idx = np.unique(knot_idx.astype(int))
-
-    return knot_idx.astype(int)
-
-
-def fit_1d_background_complex(flux, weights, wavenum, order=2, ffreq=None):
-    """Fit the background signal using a piecewise spline of n knots. Note that this will also try to identify
+def fit_1d_background_complex(flux, weights, wavenum, order=2, ffreq=None, channel=1, test=False):
+    """Fit the background signal using a pieceweise spline of n knots. Note that this will also try to identify
     obvious emission lines and flag them so they aren't considered in the fitting.
 
     :Parameters:
@@ -405,6 +410,10 @@ def fit_1d_background_complex(flux, weights, wavenum, order=2, ffreq=None):
         the expected fringe frequency, used to determine number of knots. If None,
         defaults to NUM_KNOTS constant
 
+    channel: int, optional, default=1
+        the channel processed. used to determine if other arrays need to be reversed given the direction of increasing
+        wavelength down the detector in MIRIFULONG
+
     :Returns:
 
     bg_fit: numpy array
@@ -425,6 +434,7 @@ def fit_1d_background_complex(flux, weights, wavenum, order=2, ffreq=None):
     if ffreq is not None:
         log.debug("fit_1d_background_complex: knot positions for {} cm-1".format(ffreq))
         nknots = int((np.amax(wavenum) - np.amin(wavenum)) / (ffreq))
+
     else:
         log.debug("fit_1d_background_complex: using num_knots={}".format(NUM_KNOTS))
         nknots = int((flux.shape[0] / 1024) * NUM_KNOTS)
@@ -444,25 +454,41 @@ def fit_1d_background_complex(flux, weights, wavenum, order=2, ffreq=None):
     log.debug("fit_1d_background_complex: column has {} weighted fringe periods".format(nper_cor))
 
     # require at least 5 sine periods to fit
-    if nper_cor >= 5:
-        bgindx = make_knots(flux.copy(), int(nknots), weights=weights.copy())
-        bgknots = wavenum_scaled[bgindx].astype(float)
-        bg_model = SplinesModel(bgknots, order=order)
-        fitter = Fitter(wavenum_scaled, bg_model)
-    else:
+    if nper < 5:
         log.info(" not enough weighted data, no fit performed")
         return flux.copy(), np.zeros(flux.shape[0]), None
-    ftr = RobustShell(fitter, domain=10)
-    # sometimes the fits will fail for small segments because the model is too complex for the data,
-    # in this case return the original array
-    # NOTE: since iso-alpha no longer supported this shouldn't be an issue, but will leave here for now
-    try:
-        ftr.fit(flux.copy(), weights=weights.copy())
-    except LinAlgError:
-        return flux.copy(), np.zeros(flux.shape[0]), None
+
+    bgindx = new_make_knots(flux.copy(), int(nknots), weights=weights.copy())
+    bgknots = wavenum_scaled[bgindx].astype(float)
+
+    # Reverse (and clip) the fit data as scipy/astropy need monotone increasing data for SW detector
+    if channel == 3 or channel == 4:
+        t = bgknots[1:-1]
+        x = wavenum_scaled
+        y = flux
+        w = weights
+    elif channel == 1 or channel == 2:
+        t = bgknots[::-1][1:-1]
+        x = wavenum_scaled[::-1]
+        y = flux[::-1]
+        w = weights[::-1]
+    else:
+        raise ValueError('channel not in 1-4')
+
+    # Fit the spline
+    # robust fitting causing problems for fringe 2 in channels 3 and 4, just use the fitter class
+    if ffreq > 1.5:
+        spline_model = Spline1D(knots=t, degree=2, bounds=[x[0], x[-1]])
+        fitter = SplineExactKnotsFitter()
+        robust_fitter = ChiSqOutlierRejectionFitter(fitter)
+        bg_model = robust_fitter(spline_model, x, y, weights=w)
+    else:
+        spline_model = Spline1D(knots=t, degree=1, bounds=[x[0], x[-1]])
+        fitter = SplineExactKnotsFitter()
+        bg_model = fitter(spline_model, x, y, weights=w)
 
     # fit the background
-    bg_fit = bg_model.result(wavenum_scaled)
+    bg_fit = bg_model(wavenum_scaled)
     bg_fit *= np.where(weights.copy() > 1e-07, 1, 1e-08)
 
     # linearly interpolate over the feature gaps if possible, stops issues later
@@ -473,7 +499,7 @@ def fit_1d_background_complex(flux, weights, wavenum, order=2, ffreq=None):
     except ValueError:
         pass
 
-    return bg_fit, bgindx, fitter
+    return bg_fit, bgindx
 
 
 def fit_quality(wavenum, res_fringes, weights, ffreq, dffreq, save_results=False):
@@ -522,9 +548,10 @@ def fit_quality(wavenum, res_fringes, weights, ffreq, dffreq, save_results=False
     # create the model
     mdl = SineModel(pars=[0.1, 0.1], fixed={0: 1 / peak_freq})
 
-    fitter = LevenbergMarquardtFitter(wavenum[50:-50], mdl)
+    fitter = LevenbergMarquardtFitter(wavenum[10:-10], mdl)
     ftr = RobustShell(fitter, domain=10)
-    fr_par = ftr.fit(res_fringes[50:-50], weights=weights[50:-50])
+
+    fr_par = ftr.fit(res_fringes[10:-10], weights=weights[10:-10])
     log.debug("fit_quality: best fit pars: {}".format(fr_par))
 
     if np.abs(fr_par[0]) > np.abs(fr_par[1]):
@@ -535,56 +562,42 @@ def fit_quality(wavenum, res_fringes, weights, ffreq, dffreq, save_results=False
     # make data to return for fit quality
     quality = None
     if save_results:
-        best_mdl = SineModel(fixed={0: 1 / peak_freq, 1:fr_par[0], 2:fr_par[1]})
+        best_mdl = SineModel(fixed={0: 1 / peak_freq, 1: fr_par[0], 2: fr_par[1]})
         fit = best_mdl.result(wavenum)
         quality = np.array([(10000.0 / wavenum), res_fringes, fit])
 
     return contrast, quality
 
 
-def fit_1d_fringes_bayes_evidence(res_fringes, weights, wavenum, ffreq, dffreq, min_nfringes, max_nfringes, pgram_res):
-
-    """Fit the residual fringe signal.
+def new_fit_1d_fringes_bayes_evidence(res_fringes, weights, wavenum, ffreq, dffreq, min_nfringes, max_nfringes,
+                                      pgram_res, col_snr2):
 
+    """Fit the residual fringe signal.- Improved method
     Takes an input 1D array of residual fringes and fits using the supplied mode in the BayesicFitting package:
-
     :Parameters:
-
     res_fringes:  numpy array, required
         the 1D array with residual fringes
-
     weights: numpy array, required
         the 1D array of weights
-
     ffreq: float, required
         the central scan frequency
-
     dffreq:  float, required
         the one-sided interval of scan frequencies
-
     min_nfringes: int, required
         the minimum number of fringes to check
-
     max_nfringes: int, required
         the maximum number of fringes to check
-
     pgram_res: float, optional
         resolution of the periodogram scan in cm-1
-
     wavenum: numpy array, required
         the 1D array of wavenum
-
-
     :Returns:
-
     res_fringe_fit: numpy array
         the residual fringe fit data
-
     """
     # initialize output to none
     res_fringe_fit = None
     weighted_pix_num = None
-    opt_nfringes = None
     peak_freq = None
     freq_min = None
     freq_max = None
@@ -592,7 +605,6 @@ def fit_1d_fringes_bayes_evidence(res_fringes, weights, wavenum, ffreq, dffreq,
     # get the number of weighted pixels
     weighted_pix_num = (weights > 1e-05).sum()
     # set the maximum array size, always 1024
-    max_arr_size = int(res_fringes.shape[0])
 
     # get scan res
     res = np.around((2 * dffreq) / pgram_res).astype(int)
@@ -608,126 +620,223 @@ def fit_1d_fringes_bayes_evidence(res_fringes, weights, wavenum, ffreq, dffreq,
 
     # handle out of slice pixels
     res_fringes = np.nan_to_num(res_fringes)
-    res_fringe_scan = res_fringes[np.where(weights > 1e-05)]
-    wavenum_scan = wavenum[np.where(weights > 1e-05)]
+    res_fringes[res_fringes == np.inf] = 0
+    res_fringes[res_fringes == -np.inf] = 0
 
-    # use a Lomb-Scargle periodogram to get PSD and identify the strongest frequencies
-    log.debug("fit_1d_fringes_bayes: get periodogram")
-    pgram = LombScargle(wavenum_scan[::-1], res_fringe_scan[::-1]).power(1 / freq)
+    # initialise some parameters
+    res_fringes_proc = res_fringes.copy()
+    nfringes = 0
+    keep_dict = {}
+    best_mdl = None
+    fitted_frequencies = []
 
-    res_fringe_thresh = np.mean(np.abs(pgram))
-    peaks = np.argwhere(pgram > res_fringe_thresh)[:, 0]
-    if len(peaks) == 0:
-        log.debug("fit_1d_fringes_bayes: no significant frequencies found")
-        return res_fringes, weighted_pix_num, opt_nfringes, peak_freq, freq_min, freq_max
+    # get the initial evidence from ConstantModel
+    sdml = ConstantModel(values=1.0)
+    sftr = Fitter(wavenum, sdml)
+    _ = sftr.fit(res_fringes, weights=weights)
+    evidence1 = sftr.getEvidence(limits=[-3, 10], noiseLimits=[0.001, 10])
+    log.debug(
+        "fit_1d_fringes_bayes_evidence: Initial Evidence: {}".format(evidence1))
 
-    else:
-        log.debug("fit_1d_fringes_bayes: set significant frequencies")
-        # get the peaks
-        peaks_freq = freq[peaks]
-        # peaks_power = pgram[peaks]
+    for f in np.arange(max_nfringes):
+        log.debug(
+            "Starting fringe {}".format(f + 1))
 
-        # limit to max number of fringes to MAX_NFRINGES / (weighted_pix_num / 1024), lower limit of 2
-        log.debug("fit_1d_fringes_bayes: max_array_size = {} ".format(max_arr_size))
-        log.debug("fit_1d_fringes_bayes: max_nfringes = {} ".format(max_nfringes))
-        log.debug("fit_1d_fringes_bayes: fractional weighted pixels: {}/{} ".format(weighted_pix_num, max_arr_size))
-        max_fringe_num = int(max_nfringes * (weighted_pix_num / max_arr_size))
+        # get the scan arrays
+        weights *= col_snr2
+        res_fringe_scan = res_fringes_proc[np.where(weights > 1e-05)]
+        wavenum_scan = wavenum[np.where(weights > 1e-05)]
 
-        if max_fringe_num < 2:
-            max_fringe_num = 2
+        # use a Lomb-Scargle periodogram to get PSD and identify the strongest frequency
+        log.debug("fit_1d_fringes_bayes_evidence: get the periodogram")
+        pgram = LombScargle(wavenum_scan[::-1], res_fringe_scan[::-1]).power(1 / freq)
 
-        log.debug("fit_1d_fringes_bayes: using max fringe number = {} ".format(max_fringe_num))
+        log.debug("fit_1d_fringes_bayes_evidence: get the most significant frequency in the periodogram")
+        peak = np.argmax(pgram)
+        freqs = 1. / freq[peak]
 
-        if peaks_freq.shape[0] > max_fringe_num:
-            log.debug("fit_1d_fringes_bayes: limit scan to {} strongest freqs".format(max_fringe_num))
-            peak_ind = pgram.argsort()[-max_fringe_num:][::-1]
-            freqs = 1. / freq[peak_ind]
-        else:
-            log.debug("fit_1d_fringes_bayes: limit scan to {} strongest freqs".format(peaks_freq.shape[0]))
-            peak_ind = pgram.argsort()[-peaks_freq.shape[0]:][::-1]
-            freqs = 1. / freq[peak_ind]
+        # fix the most significant frequency in the fixed dict that is passed to fitter
+        keep_ind = nfringes * 3
+        keep_dict[keep_ind] = freqs
 
-        # start from 2 frequency, add 1 each loop and check evidence
-        log.debug("fit_1d_fringes_bayes: fit {} freqs incrementally, check bayes evidence".format(freqs.shape[0]))
+        log.debug("fit_1d_fringes_bayes_evidence: creating multisine model of {} freqs".format(nfringes + 1))
+        mdl = multi_sine(nfringes + 1)
 
-        evidence1 = 1e-5  # arbitrarily small
-        opt_nfringes = min_nfringes  # initialise
-        if min_nfringes > freqs.shape[0]:
-            warning = 'Number of fringes found is less then minimum number of required fringes for column'
-            raise Exception(warning)
-
-        for nfringes in np.arange(min_nfringes, freqs.shape[0]):
-            # use the significant frequencies setup a multi-sine model of that number of sines
-            mdl_fit = multi_sine(nfringes)
-
-            # initialise some variables used in the fitting
-            pars = []
-            keep_dict = {}
-
-            # fill the variables with parameters to be frozen (freqs) and initialised (amps)
-            for n in np.arange(nfringes):
-                pars.append(freqs[n])
-                pars.append(1.0)
-                pars.append(1.0)
-                keep_index = n * 3
-                keep_dict[keep_index] = freqs[n]
-
-            # fit the multi-sine model and get evidence
-            fitter = LevenbergMarquardtFitter(wavenum, mdl_fit, verbose=0, keep=keep_dict)
+        # fit the multi-sine model and get evidence
+        if ffreq * factor > 1.5:
+            fitter = LevenbergMarquardtFitter(wavenum, mdl, verbose=0, keep=keep_dict)
             ftr = RobustShell(fitter, domain=10)
             try:
-                ftr.fit(res_fringes, weights=weights)
+                pars = ftr.fit(res_fringes, weights=weights)
+
+                # free the parameters and refit
+                mdl = multi_sine(nfringes + 1)
+                mdl.parameters = pars
+                fitter = LevenbergMarquardtFitter(wavenum, mdl, verbose=0)
+                ftr = RobustShell(fitter, domain=10)
+                pars = ftr.fit(res_fringes, weights=weights)
 
                 # try get evidence (may fail for large component fits to noisy data, set to very negative value
                 try:
-                    evidence2 = fitter.getEvidence(limits=[-1, 1], noiseLimits=[0.001, 1])
+                    evidence2 = fitter.getEvidence(limits=[-3, 10], noiseLimits=[0.001, 10])
                 except ValueError:
                     evidence2 = -1e9
-            except RuntimeError:
+            except Exception:
                 evidence2 = -1e9
 
-            log.debug("fit_1d_fringes_bayes_evidence: nfringe={} ev={} chi={}".format(nfringes, evidence2, fitter.chisq))
+        else:
+            fitter = LevenbergMarquardtFitter(wavenum, mdl, verbose=0, keep=keep_dict)
+            try:
+                pars = fitter.fit(res_fringes, weights=weights)
 
-            bayes_factor = evidence2 - evidence1
+                # free the parameters and refit
+                mdl = multi_sine(nfringes + 1)
+                mdl.parameters = pars
+                fitter = LevenbergMarquardtFitter(wavenum, mdl, verbose=0)
+                pars = fitter.fit(res_fringes, weights=weights)
+
+                # try get evidence (may fail for large component fits to noisy data, set to very negative value
+                try:
+                    evidence2 = fitter.getEvidence(limits=[-3, 10], noiseLimits=[0.001, 10])
+                except ValueError:
+                    evidence2 = -1e9
+            except Exception:
+                evidence2 = -1e9
+
+        log.debug("fit_1d_fringes_bayes_evidence: nfringe={} ev={} chi={}".format(nfringes, evidence2, fitter.chisq))
+
+        bayes_factor = evidence2 - evidence1
+        log.debug(
+            "fit_1d_fringes_bayes_evidence: bayes factor={}".format(bayes_factor))
+        if bayes_factor > 1:  # strong evidence thresh (log(bayes factor)>1, Kass and Raftery 1995)
+            evidence1 = evidence2
+            best_mdl = mdl.copy()
+            fitted_frequencies.append(freqs)
             log.debug(
-                "fit_1d_fringes_bayes_evidence: bayes factor={}".format(bayes_factor))
-            if bayes_factor > 1:  # strong evidence thresh (log(bayes factor)>1, Kass and Raftery 1995)
-                evidence1 = evidence2
-                opt_nfringes = nfringes
-                log.debug(
-                    "fit_1d_fringes_bayes_evidence: strong evidence for nfringes={} ".format(nfringes))
-            else:
-                log.debug(
-                    "fit_1d_fringes_bayes_evidence: no evidence for nfringes={}".format(nfringes))
-                break
+                "fit_1d_fringes_bayes_evidence: strong evidence for nfringes={} ".format(nfringes + 1))
+        else:
+            log.debug(
+                "fit_1d_fringes_bayes_evidence: no evidence for nfringes={}".format(nfringes + 1))
+            break
 
-        mdl_fit = multi_sine(opt_nfringes)
-        log.debug("fit_1d_fringes_bayes_evidence: optimal={} fringes".format(opt_nfringes))
+        # subtract the fringes for this frequency
+        res_fringe_fit = best_mdl(wavenum)
+        res_fringes_proc = res_fringes.copy() - res_fringe_fit
+        nfringes += 1
 
-        # initialise some variables used in the fitting
-        pars = []
-        keep_dict = {}
+    log.debug("fit_1d_fringes_bayes_evidence: optimal={} fringes".format(nfringes))
 
-        # fill the variables with parameters to be frozen (freqs) and initialised (amps)
-        for n in range(opt_nfringes):
-            pars.append(freqs[n])
-            pars.append(1.0)
-            pars.append(1.0)
-            keep_index = n * 3
-            keep_dict[keep_index] = freqs[n]
+    # create outputs to return
+    fitted_frequencies = (1 / np.asarray(fitted_frequencies)) * factor
+    peak_freq = fitted_frequencies[0]
+    freq_min = np.amin(fitted_frequencies)
+    freq_max = np.amax(fitted_frequencies)
 
-        # fit the optimal multi-sine model
-        fitter = LevenbergMarquardtFitter(wavenum, mdl_fit, verbose=0, keep=keep_dict)
-        ftr = RobustShell(fitter, domain=10)
-        fr_par = ftr.fit(res_fringes, weights=weights)
-        best_fringe_model = mdl_fit.copy()
-        best_fringe_model.parameters = fr_par
-        res_fringe_fit = best_fringe_model(wavenum)
+    return res_fringe_fit, weighted_pix_num, nfringes, peak_freq, freq_min, freq_max
 
-        # create outputs to return
-        fitted_frequencies = (1 / freqs[:opt_nfringes + 1]) * factor
-        peak_freq = fitted_frequencies[0]
-        freq_min = np.amin(fitted_frequencies)
-        freq_max = np.amax(fitted_frequencies)
 
-        return res_fringe_fit, weighted_pix_num, opt_nfringes, peak_freq, freq_min, freq_max
+def new_make_knots(flux, nknots=20, weights=None):
+    """Defines knot positions for piecewise models. This simply splits the array into sections. It does
+    NOT take into account the shape of the data.
+
+    :Parameters:
+
+    flux: numpy array, required
+        the flux array or any array of the same dimension
+
+    nknots: int, optional, default=20
+        the number of knots to create (excluding 0 and 1023)
+
+    weights: numpy array, optional, default=None
+        optionally supply a weights array. This will be used to add knots at the edge of bad pixels or features
+
+    :Returns:
+
+    knot_idx, numpy array
+        the indices of the knots
+
+    """
+    log.debug("new_make_knots: creating {} knots on flux array".format(nknots))
+
+    # handle nans or infs that may exist
+    flux = np.nan_to_num(flux, posinf=1e-08, neginf=1e-08)
+    flux[flux < 0] = 1e-08
+
+    if weights is not None:
+        weights = np.nan_to_num(weights, posinf=1e-08, neginf=1e-08)
+        weights[weights < 0] = 1e-08
+
+    # create an array of indices
+    npoints = flux.shape[0]
+
+    # kstep is the number of points / number of knots
+    knot_step = npoints / nknots
+
+    # define an initial knot index array
+    init_knot_idx = np.zeros(nknots + 1)
+    for n in range(nknots):
+        init_knot_idx[n] = round(n * knot_step)
+    init_knot_idx[-1] = npoints - 1
+
+    # get difference between the indices
+    knot_split = np.ediff1d(init_knot_idx)
+
+    # the last diff will sometimes be different than the others
+    last_split = knot_split[-1]
+
+    # create new knot array with knots at 0 and 1023, then inital and final splits = last_split/2
+    knot_idx = np.ones(nknots + 2)
+    for n in range(nknots):
+        knot_idx[n + 1] = (n * knot_step) + (last_split / 2)
+    knot_idx[0] = 0
+    knot_idx[-1] = npoints - 1
+
+    # if the weights array is supplied, determine the edges of good data and set knots there
+    if weights is not None:
+
+        log.debug("new_make_knots: adding knots at edges of bad pixels in weights array")
+
+        # if there are bad pixels in the flux array with flux~0,
+        # add these to weights array if not already there
+        weights *= (flux > 1e-03).astype(int)
+
+        # use a two-point difference method
+        weights_diff = np.ediff1d(weights)
+
+        # set edges where diff should be almost equal to the largest of the two datapoints used
+        # iterate over the diffs and compare to the datapoints
+        edges_idx_list = []
+        for n, wd in enumerate(weights_diff):
+            # get the data points used for the diff
+            datapoints = np.array([weights[n], weights[n + 1]])
+
+            # get the value and index of the larges
+            largest = np.amax(datapoints)
+            largest_idx = np.argmax(datapoints)
+
+            # we don't need knots in the bad pixels so ignore these
+            if largest > 1e-03:
+
+                # check if the absolute values are almost equal
+                if math.isclose(largest, np.abs(wd), rel_tol=1e-01):
+                    # if so, set the index and adjust depending on whether the
+                    # first or second datapoint is the largest
+                    idx = n + largest_idx
+
+                    # check if this is right next to another index already defined
+                    # causes problems in fitting, minimal difference
+                    if (idx - 1 in knot_idx) | (idx + 1 in knot_idx):
+                        pass
+                    else:
+                        # append to the index list
+                        edges_idx_list.append(idx)
+
+                else:
+                    pass
+
+        # convert the list to array, add to the knot_idx array, remove duplicates and sort
+        edges_idx = np.asarray(edges_idx_list)
+        knot_idx = np.sort(np.concatenate((knot_idx, edges_idx), axis=0), axis=0)
+        knot_idx = np.unique(knot_idx.astype(int))
+
+    return knot_idx.astype(int)
