@@ -1,4 +1,7 @@
-from pathlib import Path
+"""
+This module supports the entry points for ASDF support for the `jwst.transforms`.
+"""
+
 import sys
 
 if sys.version_info < (3, 9):
@@ -12,13 +15,20 @@ from jwst import transforms
 
 
 def get_resource_mappings():
+    """
+    Get the `jwst.transforms` resource mappings, that is the schemas for the transforms.
+
+    This method is registered with the `asdf.resource_mappings` entry point for
+    the `jwst_pipeline`.
+
+    Returns
+    -------
+    list of the `asdf.resource.ResourceMapping` instances containing the `jwst.transforms`
+    schemas.
+    """
     resources_root = importlib_resources.files(transforms) / "resources"
     if not resources_root.is_dir():
-        # In an editable install, the resources directory will exist off the
-        # repository root:
-        resources_root = Path(__file__).parent.parent.parent / "resources"
-        if not resources_root.is_dir():
-            raise RuntimeError("Missing resources directory")
+        raise RuntimeError(f"Missing resources directory: {resources_root=}")
 
     return [
         DirectoryResourceMapping(
