@@ -18,7 +18,8 @@ from ..stpipe import Step
 from .. import datamodels
 from ..assign_wcs.util import update_fits_wcsinfo
 from . import astrometric_utils as amutils
-from .tweakreg_catalog import make_tweakreg_catalog
+from . tweakreg_catalog import make_tweakreg_catalog
+from ..datamodels.util import is_association
 
 
 def _oxford_or_str_join(str_list):
@@ -50,7 +51,9 @@ class TweakRegStep(Step):
 
     spec = f"""
         save_catalogs = boolean(default=False) # Write out catalogs?
+        use_custom_catalogs = boolean(default=False) # Use custom user-provided catalogs?
         catalog_format = string(default='ecsv') # Catalog output file format
+        catfile = string(default='') # Name of the file with a list of custom user-provided catalogs
         kernel_fwhm = float(default=2.5) # Gaussian kernel FWHM in pixels
         snr_threshold = float(default=10.0) # SNR threshold above the bkg
         sharplo = float(default=0.2) # The lower bound on sharpness for object detection.
@@ -72,11 +75,8 @@ class TweakRegStep(Step):
         fitgeometry = option('shift', 'rshift', 'rscale', 'general', default='rshift') # Fitting geometry
         nclip = integer(min=0, default=3) # Number of clipping iterations in fit
         sigma = float(min=0.0, default=3.0) # Clipping limit in sigma units
-        align_to_gaia = boolean(default=False)  # Align to GAIA catalog
-        gaia_catalog = string(default='GAIADR2')  # Catalog file name or one of: {_SINGLE_GROUP_REFCAT_STR}
-        min_gaia = integer(min=0, default=5) # Min number of GAIA sources needed
-        save_gaia_catalog = boolean(default=False)  # Write out GAIA catalog as a separate product
-        output_use_model = boolean(default=True)  # When saving use `DataModel.meta.filename`
+        abs_refcat = string(default='')  # Catalog file name or one of: {_SINGLE_GROUP_REFCAT_STR}, or None, or ''
+        save_abs_catalog = boolean(default=False)  # Write out used absolute astrometric reference catalog as a separate product
         abs_minobj = integer(default=15) # Minimum number of objects acceptable for matching when performing absolute astrometry
         abs_searchrad = float(default=6.0) # The search radius in arcsec for a match when performing absolute astrometry
         # We encourage setting this parameter to True. Otherwise, xoffset and yoffset will be set to zero.
@@ -87,47 +87,112 @@ class TweakRegStep(Step):
         abs_fitgeometry = option('shift', 'rshift', 'rscale', 'general', default='rshift')
         abs_nclip = integer(min=0, default=3) # Number of clipping iterations in fit when performing absolute astrometry
         abs_sigma = float(min=0.0, default=3.0) # Clipping limit in sigma units when performing absolute astrometry
+        output_use_model = boolean(default=True)  # When saving use `DataModel.meta.filename`
     """
 
     reference_file_types = []
 
     def process(self, input):
+        use_custom_catalogs = self.use_custom_catalogs
+
+        if use_custom_catalogs:
+            catdict = _parse_catfile(self.catfile)
+            # if user requested the use of custom catalogs and provided a
+            # valid 'catfile' file name that has no custom catalogs,
+            # turn off the use of custom catalogs:
+            if catdict is not None and not catdict:
+                self.log.warning(
+                    "'use_custom_catalogs' is set to True but 'catfile' "
+                    "contains no user catalogs. Turning on built-in catalog "
+                    "creation."
+                )
+                use_custom_catalogs = False
 
         try:
-            images = datamodels.ModelContainer(input)
+            if use_custom_catalogs and catdict:
+                images = datamodels.ModelContainer()
+                if isinstance(input, str):
+                    asn_dir = path.dirname(input)
+                    asn_data = images.read_asn(input)
+                    for member in asn_data['products'][0]['members']:
+                        filename = member['expname']
+                        member['expname'] = path.join(asn_dir, filename)
+                        if filename in catdict:
+                            member['tweakreg_catalog'] = catdict[filename]
+                        elif 'tweakreg_catalog' in member:
+                            del member['tweakreg_catalog']
+
+                    images.from_asn(input)
+
+                elif is_association(input):
+                    images.from_asn(input)
+
+                else:
+                    images = datamodels.ModelContainer(input)
+                    for im in images:
+                        filename = im.meta.filename
+                        if filename in catdict:
+                            print(f"setting {filename}.tweakreg_catalog = {repr(catdict[filename])}")
+                            im.meta.tweakreg_catalog = catdict[filename]
+
+            else:
+                images = datamodels.ModelContainer(input)
+
         except TypeError as e:
             e.args = ("Input to tweakreg must be a list of DataModels, an "
                       "association, or an already open ModelContainer "
                       "containing one or more DataModels.", ) + e.args[1:]
             raise e
 
-        if self.align_to_gaia:
+        if self.abs_refcat is not None and self.abs_refcat.strip():
+            align_to_abs_refcat = True
             # Set expand_refcat to True to eliminate possibility of duplicate
-            # entries when aligning to GAIA
+            # entries when aligning to absolute astrometric reference catalog
             self.expand_refcat = True
+        else:
+            align_to_abs_refcat = False
 
         if len(images) == 0:
             raise ValueError("Input must contain at least one image model.")
 
         # Build the catalogs for input images
         for image_model in images:
-            # source finding
-            catalog = make_tweakreg_catalog(
-                image_model, self.kernel_fwhm, self.snr_threshold,
-                sharplo=self.sharplo, sharphi=self.sharphi,
-                roundlo=self.roundlo, roundhi=self.roundhi,
-                brightest=self.brightest, peakmax=self.peakmax,
-                bkg_boxsize=self.bkg_boxsize
-            )
+            if use_custom_catalogs and image_model.meta.tweakreg_catalog:
+                # use user-supplied catalog:
+                self.log.info("Using user-provided input catalog "
+                              f"'{image_model.meta.tweakreg_catalog}'")
+                catalog = Table.read(image_model.meta.tweakreg_catalog)
+                new_cat = False
+
+            else:
+                # source finding
+                catalog = make_tweakreg_catalog(
+                    image_model, self.kernel_fwhm, self.snr_threshold,
+                    sharplo=self.sharplo, sharphi=self.sharphi,
+                    roundlo=self.roundlo, roundhi=self.roundhi,
+                    brightest=self.brightest, peakmax=self.peakmax,
+                    bkg_boxsize=self.bkg_boxsize
+                )
+                new_cat = True
+
+            for axis in ['x', 'y']:
+                if axis not in catalog.colnames:
+                    long_axis = axis + 'centroid'
+                    if long_axis in catalog.colnames:
+                        catalog.rename_column(long_axis, axis)
+                    else:
+                        raise ValueError(
+                            "'tweakreg' source catalogs must contain either "
+                            "columns 'x' and 'y' or 'xcentroid' and "
+                            "'ycentroid'."
+                        )
 
             # filter out sources outside the WCS bounding box
             bb = image_model.meta.wcs.bounding_box
             if bb is not None:
                 ((xmin, xmax), (ymin, ymax)) = bb
-                xname = 'xcentroid' if 'xcentroid' in catalog.colnames else 'x'
-                yname = 'ycentroid' if 'ycentroid' in catalog.colnames else 'y'
-                x = catalog[xname]
-                y = catalog[yname]
+                x = catalog['x']
+                y = catalog['y']
                 mask = (x > xmin) & (x < xmax) & (y > ymin) & (y < ymax)
                 catalog = catalog[mask]
 
@@ -139,7 +204,7 @@ class TweakRegStep(Step):
                 self.log.info('Detected {} sources in {}.'
                               .format(len(catalog), filename))
 
-            if self.save_catalogs:
+            if new_cat and self.save_catalogs:
                 catalog_filename = filename.replace(
                     '.fits', '_cat.{}'.format(self.catalog_format)
                 )
@@ -170,7 +235,7 @@ class TweakRegStep(Step):
                       .format(len(grp_img)))
         self.log.info("Image groups:")
 
-        if len(grp_img) == 1 and not self.align_to_gaia:
+        if len(grp_img) == 1 and not align_to_abs_refcat:
             self.log.info("* Images in GROUP 1:")
             for im in grp_img[0]:
                 self.log.info("     {}".format(im.meta.filename))
@@ -242,7 +307,7 @@ class TweakRegStep(Step):
                     self.log.warning("Nothing to do. Skipping 'TweakRegStep'...")
                     for model in images:
                         model.meta.cal_step.tweakreg = "SKIPPED"
-                    if not self.align_to_gaia:
+                    if not align_to_abs_refcat:
                         self.skip = True
                         return images
                 else:
@@ -277,22 +342,22 @@ class TweakRegStep(Step):
 
                     for model in images:
                         model.meta.cal_step.tweakreg = "SKIPPED"
-                    if self.align_to_gaia:
+                    if align_to_abs_refcat:
                         self.log.warning("Skipping relative alignment (stage 1)...")
                     else:
                         self.log.warning("Skipping 'TweakRegStep'...")
                         self.skip = True
                         return images
 
-        if self.align_to_gaia:
+        if align_to_abs_refcat:
             # Get catalog of GAIA sources for the field
             #
             # NOTE:  If desired, the pipeline can write out the reference
             #        catalog as a separate product with a name based on
             #        whatever convention is determined by the JWST Cal Working
             #        Group.
-            if self.save_gaia_catalog:
-                output_name = 'fit_{}_ref.ecsv'.format(self.gaia_catalog.lower())
+            if self.save_abs_catalog:
+                output_name = 'fit_{}_ref.ecsv'.format(self.abs_refcat.lower())
             else:
                 output_name = None
 
@@ -300,8 +365,8 @@ class TweakRegStep(Step):
             self.abs_xoffset = 0
             self.abs_yoffset = 0
 
-            self.gaia_catalog = self.gaia_catalog.strip()
-            gaia_cat_name = self.gaia_catalog.upper()
+            self.abs_refcat = self.abs_refcat.strip()
+            gaia_cat_name = self.abs_refcat.upper()
 
             if gaia_cat_name in SINGLE_GROUP_REFCAT:
                 ref_cat = amutils.create_astrometric_catalog(
@@ -310,22 +375,22 @@ class TweakRegStep(Step):
                     output=output_name
                 )
 
-            elif path.isfile(self.gaia_catalog):
-                ref_cat = Table.read(self.gaia_catalog)
+            elif path.isfile(self.abs_refcat):
+                ref_cat = Table.read(self.abs_refcat)
 
             else:
-                raise ValueError("'gaia_catalog' must be a path to an "
+                raise ValueError("'abs_refcat' must be a path to an "
                                  "existing file name or one of the supported "
                                  f"reference catalogs: {_SINGLE_GROUP_REFCAT_STR}.")
 
             # Check that there are enough GAIA sources for a reliable/valid fit
             num_ref = len(ref_cat)
-            if num_ref < self.min_gaia:
+            if num_ref < self.abs_minobj:
                 # Raise Exception here to avoid rest of code in this try block
                 self.log.warning(
                     f"Not enough sources ({num_ref}) in the reference catalog "
                     "for the single-group alignment step to perform a fit. "
-                    f"Skipping alignment to the {self.gaia_catalog} reference "
+                    f"Skipping alignment to the {self.abs_refcat} reference "
                     "catalog!"
                 )
             else:
@@ -375,7 +440,7 @@ class TweakRegStep(Step):
                 # Update/create the WCS .name attribute with information
                 # on this astrometric fit as the only record that it was
                 # successful:
-                if self.align_to_gaia:
+                if align_to_abs_refcat:
                     # NOTE: This .name attrib agreed upon by the JWST Cal
                     #       Working Group.
                     #       Current value is merely a place-holder based
@@ -383,7 +448,7 @@ class TweakRegStep(Step):
                     #       translated to the FITS WCSNAME keyword
                     #       IF that is what gets recorded in the archive
                     #       for end-user searches.
-                    imcat.wcs.name = "FIT-LVL3-{}".format(self.gaia_catalog)
+                    imcat.wcs.name = "FIT-LVL3-{}".format(self.abs_refcat)
 
                 image_model.meta.wcs = imcat.wcs
 
@@ -392,7 +457,8 @@ class TweakRegStep(Step):
                 try:
                     update_fits_wcsinfo(
                         image_model,
-                        max_pix_error=0.005
+                        max_pix_error=0.01,
+                        npoints=16
                     )
                 except (ValueError, RuntimeError) as e:
                     self.log.warning(
@@ -436,10 +502,6 @@ class TweakRegStep(Step):
             except IOError:
                 self.log.error("Cannot read catalog {}".format(catalog))
 
-        if 'xcentroid' in catalog.colnames:
-            catalog.rename_column('xcentroid', 'x')
-            catalog.rename_column('ycentroid', 'y')
-
         # create WCSImageCatalog object:
         refang = image_model.meta.wcsinfo.instance
         im = JWSTWCSCorrector(
@@ -462,3 +524,29 @@ def _common_name(group):
     cn = path.commonprefix(file_names)
     assert cn
     return cn
+
+
+def _parse_catfile(catfile):
+    if catfile is None or not catfile.strip():
+        return None
+
+    catdict = {}
+
+    with open(catfile) as f:
+        catfile_dir = path.dirname(catfile)
+
+        for line in f.readlines():
+            sline = line.strip()
+            if not sline or sline[0] == '#':
+                continue
+
+            data_model, *catalog = sline.split()
+            catalog = list(map(str.strip, catalog))
+            if len(catalog) == 1:
+                catdict[data_model] = path.join(catfile_dir, catalog[0])
+            elif len(catalog) == 0:
+                catdict[data_model] = None
+            else:
+                raise ValueError("'catfile' can contain at most two columns.")
+
+    return catdict
