@@ -1,9 +1,8 @@
 import logging
 
 import numpy as np
+import scipy.signal
 from scipy.interpolate import griddata
-from scipy.signal import convolve
-from scipy import mgrid
 
 from .. import datamodels
 from ..datamodels import dqflags
@@ -12,13 +11,6 @@ from ..datamodels import dqflags
 log = logging.getLogger(__name__)
 log.setLevel(logging.DEBUG)
 
-# The following should probably be in a ref file instead
-OFF_DELTA = 2  # for searching +/-2 around nominal offsets
-PSF_SIZE = 200  # half width of psf for 12 waves
-BLUR_SIZE = 10  # size of gaussian kernel for convolution
-N_SIZE = 2  # size of neighborhood in create_griddata_array
-
-
 DO_NOT_USE = dqflags.pixel["DO_NOT_USE"]
 
 
@@ -28,7 +20,8 @@ class DataSet:
 
     """
 
-    def __init__(self, infile_1, infile_2, outfile, do_refine):
+    def __init__(self, infile_1, infile_2, outfile, do_refine, flip_dithers, psf_size,
+                 blur_size, n_size):
         """
         Short Summary
         -------------
@@ -44,6 +37,14 @@ class DataSet:
             File for combined image
         do_refine: boolean
             True if refined offset calculation and application is to be made
+        flip_dithers: boolean
+            True will cause the dithers to align in pixel coordinates for different filters
+        psf_size: float
+            size of largest psf
+        blur_size: float
+            amount of smoothing to apply before finding the initial centroid
+        n_size: int
+            size of interpolation box
         """
 
         if outfile == "":
@@ -58,13 +59,23 @@ class DataSet:
 
         self.do_refine = do_refine
 
-        if (self.input_1.data.shape != self.input_2.data.shape):
+        if self.input_1.data.shape != self.input_2.data.shape:
             log.error('Incompatible sizes for input files')
 
-        log.info('File 1 to combine: %s', infile_1)
-        log.info('File 2 to combine: %s', infile_2)
         log.info('Output file: %s', outfile)
         log.info('do_refine: %s', do_refine)
+        log.info('flip_dithers: %s', flip_dithers)
+        self.file1 = infile_1
+        self.file2 = infile_2
+        self.off_x = 0
+        self.off_y = 0
+        self.flt_off_x = 0
+        self.flt_off_y = 0
+        self.diff = np.ndarray(shape=(2 * psf_size + 1, 2 * psf_size + 1))
+        self.flip_dithers = flip_dithers
+        self.psf_size = psf_size
+        self.blur_size = blur_size
+        self.n_size = n_size
 
     def do_all(self):
         """
@@ -81,21 +92,42 @@ class DataSet:
             combined input file data
         """
 
+        self.off_x, self.off_y = self.get_wcs_offsets()
+
+        # If the shift in x is negative, switch the two images
+        if self.off_x < 0 and self.flip_dithers:
+            self.input_1.close()
+            self.input_2.close()
+            self.input_1 = datamodels.open(self.file2)
+            self.input_2 = datamodels.open(self.file1)
+            log.info('File 1 to combine: %s', self.file2)
+            log.info('File 2 to combine: %s', self.file1)
+        else:
+            log.info('File 1 to combine: %s', self.file1)
+            log.info('File 2 to combine: %s', self.file2)
+
         # Input SCI arrays may have nan's so replace with 0's to prevent
         # later annoyances (hopefully this can be removed later)
         self.input_1.data[np.isnan(self.input_1.data)] = 0.
         self.input_2.data[np.isnan(self.input_2.data)] = 0.
-
         im_1_a = self.input_1.copy()  # Aligned image #1 (already aligned)
         im_2_a = self.create_aligned_2()  # Aligned image #2
 
         # Create and populate extensions for combined data
-        data_c, dq_c, err_c = self.create_combined(im_1_a, im_2_a)
+        data_c, dq_c, err_c, diff_c = self.create_combined(im_1_a, im_2_a)
 
+        log.info(f"Final x, y offset in pixels: {self.off_x} {self.off_y}")
+
+        self.diff = diff_c
         # Create a new model using the combined arrays...
         new_model = datamodels.ImageModel(data=data_c, dq=dq_c, err=err_c)
         new_model.update(self.input_1)
-
+        new_model.history.append('Flip dithers = {}'.format(self.flip_dithers))
+        new_model.history.append('WFS_COMBINE refine offset = {}'.format(self.do_refine))
+        new_model.history.append('WFS_COMBINE X offset applied ' + str(self.off_x) + ' pixels ' +
+                                 'actual offset ' + str(round(self.flt_off_x, 2)) + ' pixels')
+        new_model.history.append('WFS_COMBINE Y offset applied ' + str(self.off_y) + ' pixels ' +
+                                 'actual offset ' + str(round(self.flt_off_y, 2)) + ' pixels')
         return new_model
 
     def create_aligned_2(self):
@@ -116,7 +148,7 @@ class DataSet:
            smoothed image, and taking the mean of the coordinates of these
            pixels.  The x- and y-mean defines the centroid location.
         3. Set the limits of the subarrays for interpolation by taking this
-           centroid +/- PSF_SIZE and adding the BLUR_SIZE, taking the edges
+           centroid +/- psf_size and adding the BLUR_SIZE, taking the edges
            into account.
         4. Determine overlap of these interpolated images, and return nominally
            aligned, interpolated images.
@@ -134,7 +166,7 @@ class DataSet:
         """
 
         self.off_x, self.off_y = self.get_wcs_offsets()
-
+        log.info(f"x,y offset in integer pixels from WCS: {self.off_x} {self.off_y}")
         if self.do_refine:
             # 1. Create smoothed image of input SCI data of image #1
             # 1a. create image to smooth by first setting bad DQ pixels equal
@@ -144,60 +176,55 @@ class DataSet:
             data_1[bad1] = data_1[~bad1].mean()
 
             # 1b. Create smoothed image by smoothing this 'repaired' image
-            g = gauss_kern(BLUR_SIZE, sizey=None)
-            s_data_1 = convolve(data_1, g, mode='valid')
+            g = gauss_kern(self.blur_size, sizey=None)
+            s_data_1 = scipy.signal.convolve(data_1, g, mode='valid')
 
-            # 2. Find approximate center of PSF in umsmoothed frame by taking
+            # 2. Find approximate center of PSF in unsmoothed frame by taking
             #    all pixels in smoothed image exceeding 50% of the maximum
             #    of the smoothed image, and taking the mean of the coordinates
             #    of these pixels. Add BLUR_SIZE to take smoothing into account
             wh_data_hi = np.where(s_data_1 > 0.5 * s_data_1.max())
 
-            ctrd_x = wh_data_hi[1].mean() + BLUR_SIZE
-            ctrd_y = wh_data_hi[0].mean() + BLUR_SIZE
+            ctrd_x = wh_data_hi[1].mean() + self.blur_size
+            ctrd_y = wh_data_hi[0].mean() + self.blur_size
 
             log.info('Approximate centroid of image 1 PSF has x,y : %s %s',
                      round(ctrd_x), round(ctrd_y))
 
             # 3. Set limits of the subarrays (in frames of input data)
-            #    for interpolation by taking this centroid +/- PSF_SIZE
+            #    for interpolation by taking this centroid +/- psf_size
             #    and adding BLUR_SIZE, taking edges into account
-            xmin = int(round(max(0, ctrd_x - PSF_SIZE)))
-            ymin = int(round(max(0, ctrd_y - PSF_SIZE)))
-            xmax = int(round(min(self.input_1.data.shape[1], ctrd_x + PSF_SIZE)))
-            ymax = int(round(min(self.input_1.data.shape[0], ctrd_y + PSF_SIZE)))
+            xmin = int(round(max(0, ctrd_x - self.psf_size)))
+            ymin = int(round(max(0, ctrd_y - self.psf_size)))
+            xmax = int(round(min(self.input_1.data.shape[1], ctrd_x + self.psf_size)))
+            ymax = int(round(min(self.input_1.data.shape[0], ctrd_y + self.psf_size)))
 
             # 3a. Set subarrays and interpolate over bad pixels
             data_sub_1 = self.input_1.data[ymin: ymax, xmin: xmax]
             dq_sub_1 = self.input_1.dq[ymin: ymax, xmin: xmax]
-            sci_int_1 = interp_array(data_sub_1, dq_sub_1)
+            sci_int_1 = interp_array(data_sub_1, dq_sub_1, self.n_size)
 
             data_sub_2 = self.input_2.data[ymin: ymax, xmin: xmax]
             dq_sub_2 = self.input_2.dq[ymin: ymax, xmin: xmax]
-            sci_int_2 = interp_array(data_sub_2, dq_sub_2)
+            sci_int_2 = interp_array(data_sub_2, dq_sub_2, self.n_size)
 
             # 4. Determine overlap of these interpolated images, and
             #    return nominally aligned, interpolated images
             sci_nai_1, sci_nai_2 = get_overlap(sci_int_1, sci_int_2,
                                                self.off_x, self.off_y)
-
             # 5. Around this nominal alignment, get refined (delta) offsets
-            ref_del_off_x, ref_del_off_y = optimize_offs(sci_nai_1, sci_nai_2)
-
+            ref_del_off_x, ref_del_off_y = calc_refined_offsets(sci_nai_1, sci_nai_2, 0, 0, self.psf_size)
             log.info('From the refined offsets calculation,'
-                     'the x,y changes in ofsets are: %s %s',
-                     ref_del_off_x, ref_del_off_y)
+                     'the x,y changes in offsets are: %s %s',
+                     round(ref_del_off_x, 2), round(ref_del_off_y, 2))
 
             # 6. Add the refined delta offsets to the nominal offsets
-            self.off_x += ref_del_off_x
-            self.off_y += ref_del_off_y
+            self.flt_off_x = self.off_x + ref_del_off_x
+            self.flt_off_y = self.off_y + ref_del_off_y
+            self.off_x += int(round(ref_del_off_x))
+            self.off_y += int(round(ref_del_off_y))
 
-            log.info('Values for the refined offsets are, for x,y : %s %s',
-                     self.off_x, self.off_y)
-
-        log.info(f"Final x,y offset in pixels: {self.off_x} {self.off_y}")
-
-        # Do final alignment for original (not interpolated) image #2
+        # Do the final alignment for original (not interpolated) image two
         data_2_a, dq_2_a, err_2_a = self.apply_final_offsets()
 
         model_2_a = self.input_2.copy()  # Model for aligned image #2
@@ -265,7 +292,10 @@ class DataSet:
         pixels = tr2(radec[0], radec[1])
         off_x = pixels[0] - xcen
         off_y = pixels[1] - ycen
-
+        log.info('From the WCS the x,y pixel offsets are: %s %s',
+                 round(off_x, 2), round(off_y, 2))
+        self.flt_off_x = off_x
+        self.flt_off_y = off_y
         off_x = int(round(off_x))  # Offsets required to be integers
         off_y = int(round(off_y))
 
@@ -345,7 +375,12 @@ class DataSet:
         err_comb[good1 & bad2] = err1[good1 & bad2]
         err_comb[good2 & bad1] = err2[good2 & bad1]
 
-        return data_comb, dq_comb, err_comb
+        data_diff = np.zeros_like(data1)
+        data_diff[good1 & good2] = (data1[good1 & good2] - data2[good1 & good2])
+        data_diff[good1 & bad2] = 0
+        data_diff[good2 & bad1] = 0
+
+        return data_comb, dq_comb, err_comb, data_diff
 
     def do_2d_shifts(self, a):
         """
@@ -353,20 +388,18 @@ class DataSet:
         -------------
         Create 2d output array by shifting 2d array input by (off_x, off_y),
         where output has same dimensions as input.
-
         Parameters
         ----------
         a: 2d float array
             input array
-
         Returns
         -------
         b: 2d float array
             shifted array of input a
         """
 
-        ai_x, af_x = get_index_range(self.off_x, a.shape[1])
-        ai_y, af_y = get_index_range(self.off_y, a.shape[0])
+        ai_x, af_x = get_final_index_range(self.off_x, a.shape[1])
+        ai_y, af_y = get_final_index_range(self.off_y, a.shape[0])
 
         bi_x = a.shape[1] - af_x  # For output, x-direction's initial channel
         bf_x = a.shape[1] - ai_x  # ...and final channel
@@ -380,6 +413,33 @@ class DataSet:
         return b
 
 
+def get_final_index_range(offset, length):
+    """
+    Short Summary
+    -------------
+    Get the initial and final indices for the given offset and array length:
+    For offset <= 0: i = 0,  f = length - abs(offset)
+    For offset > 0: i = offset,  f = length
+    Parameters
+    ----------
+    offset: integer
+        offset
+    length: integer
+        length of (1D) array
+    Returns
+    -------
+    i: integer
+        initial index
+    f: integer
+        final index
+    """
+
+    i = int((abs(offset) + offset) / 2)
+    f = length + int((-abs(offset) + offset) / 2)
+
+    return i, f
+
+
 def gauss_kern(size, sizey=None):
     """
     Short Summary
@@ -405,13 +465,13 @@ def gauss_kern(size, sizey=None):
     else:
         sizey = int(sizey)
 
-    x, y = mgrid[-size:size + 1, -sizey:sizey + 1]
+    x, y = np.mgrid[-size:size + 1, -sizey:sizey + 1]
     g = np.exp(-(x**2 / float(size) + y**2 / float(sizey)))
 
     return g / g.sum()
 
 
-def interp_array(sci_data, dq_data):
+def interp_array(sci_data, dq_data, n_size):
     """
     Short Summary
     -------------
@@ -425,6 +485,8 @@ def interp_array(sci_data, dq_data):
         original SCI image to interpolate over
     dq_data: 2D int array
         corresponding DQ image
+    n_size: int
+        size of the interpolation box
 
     Returns
     -------
@@ -442,7 +504,7 @@ def interp_array(sci_data, dq_data):
 
     # Loop over bad pixels, filling in missing values with interpolated values
     for jj in np.arange(num_bad_dq):
-        ga = create_griddata_array(sci_data, bad_dq[jj])
+        ga = create_griddata_array(sci_data, bad_dq[jj], n_size)
 
         x = bad_dq[jj][1]
         y = bad_dq[jj][0]
@@ -458,7 +520,7 @@ def interp_array(sci_data, dq_data):
     return sci_data
 
 
-def create_griddata_array(sci_data, pixel):
+def create_griddata_array(sci_data, pixel, n_size):
     """
     Short Summary
     -------------
@@ -474,6 +536,9 @@ def create_griddata_array(sci_data, pixel):
     pixel: int, int
         y, x coordinates of pixel to interpolate over
 
+    n_size: int
+        size of the interpolation box
+
     Returns
     -------
     interp_arr: int, int, float
@@ -484,10 +549,10 @@ def create_griddata_array(sci_data, pixel):
     ydim = sci_data.shape[0]
 
     # Generate neighborhood limits
-    xmin = max(0, pixel[1] - N_SIZE)
-    ymin = max(0, pixel[0] - N_SIZE)
-    xmax = min(xdim - N_SIZE, pixel[1] + N_SIZE)
-    ymax = min(ydim - N_SIZE, pixel[0] + N_SIZE)
+    xmin = max(0, pixel[1] - n_size)
+    ymin = max(0, pixel[0] - n_size)
+    xmax = min(xdim - n_size, pixel[1] + n_size)
+    ymax = min(ydim - n_size, pixel[0] + n_size)
 
     # Make a list for neighboring pixels, containing:
     # 1. coordinates for up to (2*N_SIZE+1)^2-1 neighbors, accounting for edges
@@ -527,16 +592,21 @@ def get_index_range(offset, length):
 
     Returns
     -------
-    i: integer
-        initial index
-    f: integer
-        final index
+        initial index : integer
+        final index : integer
     """
+    if offset > 0:
+        initial_1 = 0
+        final_1 = length - offset
+        initial_2 = offset
+        final_2 = length
+    else:
+        initial_1 = abs(offset)
+        final_1 = length
+        initial_2 = 0
+        final_2 = length - abs(offset)
 
-    i = int((abs(offset) + offset) / 2)
-    f = length + int((-abs(offset) + offset) / 2)
-
-    return i, f
+    return initial_1, final_1, initial_2, final_2
 
 
 def get_overlap(sci_int_1, sci_int_2, nom_off_x, nom_off_y):
@@ -558,15 +628,15 @@ def get_overlap(sci_int_1, sci_int_2, nom_off_x, nom_off_y):
     a given offset 'off', the resulting initial and final indices, and
     the elements of arrays indexed are:
 
-    for off < 0 : ix = 0 and fx = length - abs(off)
+    for off < 0 : ix = 0 and final_x = length - abs(off)
        subarray indices: sub_1[0: length - abs(off)]
        subarray indices: sub_2[0: length - abs(off)]
 
-    for off = 0 : ix = 0 ; fx = length)
+    for off = 0 : ix = 0 ; final_x = length)
        subarray indices: sub_1[0: length]
        subarray indices: sub_2[0: length]
 
-    for off > 0 : ix = off ; fx = length
+    for off > 0 : ix = off ; final_x = length
        subarray indices: sub_1[0: length - off]
        subarray indices: sub_2[off: length]
 
@@ -591,60 +661,25 @@ def get_overlap(sci_int_1, sci_int_2, nom_off_x, nom_off_y):
 
     # From the nominal offsets, determine array indices to shift image #2
     #     onto frame #1
-    ix, fx = get_index_range(nom_off_x, sci_int_2.shape[1])
-    iy, fy = get_index_range(nom_off_y, sci_int_2.shape[0])
+    initial_x_1, final_x_1, initial_x_2, final_x_2 = get_index_range(nom_off_x, sci_int_2.shape[1])
+    initial_y_1, final_y_1, initial_y_2, final_y_2 = get_index_range(nom_off_y, sci_int_2.shape[0])
 
-    sub_1 = sci_int_1[:fy - iy, :fx - ix]
-    sub_2 = sci_int_2[iy:fy, ix:fx]
+    sub_1 = sci_int_1[initial_y_1:final_y_1, initial_x_1:final_x_1]
+    sub_2 = sci_int_2[initial_y_2:final_y_2, initial_x_2:final_x_2]
 
     return sub_1, sub_2
 
 
-def optimize_offs(sci_nai_1, sci_nai_2):
-    """
-    Short Summary
-    -------------
-    Calculate refined values for the offsets. Find the nearby integer
-    relative offsets between the nominally aligned interpolated images
-    which minimizes the square of the difference between the 2 images.
-
-    Parameters
-    ----------
-    sci_nai_1: 2d float array
-        nominally aligned, interpolated SCI array for image 1
-    sci_nai_2: 2d float array
-        nominally aligned, interpolated SCI array for image 2
-
-    Returns
-    -------
-    max_off_x: integer
-        offset in x-direction which minimizes array difference
-    max_off_y: integer
-        offset in y-direction which minimizes array difference
-
-    """
-
-    max_diff = 0.
-
-    for off_y in range(-OFF_DELTA, OFF_DELTA + 1):
-        for off_x in range(-OFF_DELTA, OFF_DELTA + 1):
-            this_diff = calc_cor_coef(sci_nai_1, sci_nai_2, off_x, off_y)
-
-            if (this_diff > max_diff):
-                max_diff = this_diff
-                max_off_x = off_x
-                max_off_y = off_y
-
-    return max_off_x, max_off_y
-
-
-def calc_cor_coef(sci_nai_1, sci_nai_2, off_x, off_y):
+def calc_refined_offsets(sci_nai_1, sci_nai_2, off_x, off_y, psf_size):
     """
     Short Summary
     -------------
     Get the overlap of the 2 images (based on the offsets), and
-    calculate the correlation coefficient between 2 image subarrays.
-    (We may want something more sophisticated later)
+    calculate the two dimensional cross correlation image between 2 image subarrays.
+    Then we slice on the a subarray around the peak of the cross correlation image and
+    find the first moment. The first moment provides a high S/N measurement of the offset
+    between the two images.
+
 
     Parameters
     ----------
@@ -656,19 +691,23 @@ def calc_cor_coef(sci_nai_1, sci_nai_2, off_x, off_y):
         offset in x-direction
     off_y: integer
         offset in y-direction
+    psf_size: integer
+        The worst case PSF size
 
     Returns
     -------
-    mean_diff: float
-        mean of correlation coefficient array
+    refined_x: float
+        The refined value of the x offset
+    refined_y: float
+        The refined value of the y offset
 
     """
-
+    centroid_size = 3
     sub_1, sub_2 = get_overlap(sci_nai_1, sci_nai_2, off_x, off_y)
     num_pix = sub_1.shape[0] * sub_1.shape[1]  # Number of overlapping pixels
 
     # Raise (fatal) exception if there are no overlapping pixels
-    if (num_pix == 0):
+    if num_pix == 0:
         log.error('Applying offsets to image #2 results in 0 overlapping pix')
         raise RuntimeWarning('No overlapping pixels in 2 images')
 
@@ -679,18 +718,26 @@ def calc_cor_coef(sci_nai_1, sci_nai_2, off_x, off_y):
     xcen = xlen / 2 + 1
     ycen = ylen / 2 + 1
 
-    xmin = int(max(0, xcen - PSF_SIZE))
-    xmax = int(min(xlen - 1, xcen + PSF_SIZE))
-    ymin = int(max(0, ycen - PSF_SIZE))
-    ymax = int(min(ylen - 1, ycen + PSF_SIZE))
+    xmin = int(max(0, xcen - psf_size))
+    xmax = int(min(xlen - 1, xcen + psf_size))
+    ymin = int(max(0, ycen - psf_size))
+    ymax = int(min(ylen - 1, ycen + psf_size))
 
-    # Create subarrays using these limits, and make them
-    #   1D for numpy's correlation coefficient
+    # Create subarrays using these limits
     sub_1_sub = sub_1[ymin:ymax, xmin:xmax]
     sub_2_sub = sub_2[ymin:ymax, xmin:xmax]
-    sub_1_sub = np.ravel(sub_1_sub)
-    sub_2_sub = np.ravel(sub_2_sub)
+    # Create the cross correlation image
+    cross_cor = scipy.signal.correlate2d(sub_2_sub - scipy.ndimage.gaussian_filter(sub_2_sub, 5), sub_1_sub -
+                                         scipy.ndimage.gaussian_filter(sub_1_sub, 5))
+    maximum_pixel = np.unravel_index(np.argmax(cross_cor), cross_cor.shape)
 
-    cor_coef = np.corrcoef(sub_1_sub, sub_2_sub)
-
-    return cor_coef.mean()
+    ymax = maximum_pixel[0] - sub_1_sub.shape[0] + 1
+    xmax = maximum_pixel[1] - sub_1_sub.shape[1] + 1
+    # Slice out a box center on the peak of the cross correlation image. The centroid of this box will give a
+    # accurate estimate of the x and y offsets.
+    central_cutout = cross_cor[maximum_pixel[0] - centroid_size:maximum_pixel[0] + centroid_size + 1,
+                               maximum_pixel[1] - centroid_size:maximum_pixel[1] + centroid_size + 1]
+    centroid = scipy.ndimage.measurements.center_of_mass(central_cutout)
+    refined_x = xmax + centroid[1] - centroid_size
+    refined_y = ymax + centroid[0] - centroid_size
+    return refined_x, refined_y
